from . import example_filetype_format

BASE_CLASS = example_filetype_format.FileTypeFormat

def make_format_registry_dict(cls_list):
    """Use an object's _fileType attribute to make a class lookup dictionary.
    
    Args:
        cls_list: A list of Python classes.
    Returns:
        A dictionary mapping the class._fileType to the class.
    """

    return {cls._fileType: cls for cls in cls_list}

def get_subclasses(cls):
    for subclass in cls.__subclasses__():
        yield from get_subclasses(subclass)
        yield subclass

PROCESS_FILES_LIST = [x for x in get_subclasses(BASE_CLASS)]

PROCESS_FILES = make_format_registry_dict(cls_list=PROCESS_FILES_LIST)



import logging
import os

import pandas as pd

from .sampleRetraction import sampleRetraction

logger = logging.getLogger(__name__)

class patientRetraction(sampleRetraction):

    _fileType = "patientRetraction"


#! /usr/bin/env python
import argparse

import synapseclient

from . import process_functions

def retract_samples(syn, database_synid, col, remove_values):
    '''
    Helper retraction function that help remove values from a column in
    a database

    params:
        syn: synapse object
        databse_synid:  synapse id of database
        col: column in database
        remove_values: list of values to remove from the column 
    '''

    schema = syn.get(database_synid)
    remove_values_query = "','".join(remove_values)
    remove_rows = syn.tableQuery("select %s from %s where %s in ('%s')" % (col, database_synid, col, remove_values_query))
    if len(remove_rows.asRowSet()['rows']) > 0:
        syn.delete(remove_rows.asRowSet())
    else:
        print("Nothing to retract")


def retract(syn, test=False):
    '''
    Main retraction function
    
    params:
        syn: synapse object
        test: use test files or main files. Default is False
    '''

    patientRetract = syn.tableQuery('select * from %s' % process_functions.getDatabaseSynId(syn, "patientRetraction", test=test))
    patientRetractIds = patientRetract.asDataFrame()
    #grab all clinical samples that belong to patients in the patient clinical file and append to sample list
    sampleClinical = syn.tableQuery('select * from %s' % process_functions.getDatabaseSynId(syn, "sample", test=test))
    sampleClinicalDf = sampleClinical.asDataFrame()
    appendSamples = sampleClinicalDf['SAMPLE_ID'][sampleClinicalDf['PATIENT_ID'].isin(patientRetractIds.geniePatientId)]
    
    sampleRetract = syn.tableQuery('select * from %s' % process_functions.getDatabaseSynId(syn, "sampleRetraction", test=test))
    sampleRetractIds = sampleRetract.asDataFrame()

    allRetractedSamples = sampleRetractIds['genieSampleId'].append(appendSamples)

    #Only need to retract clinical data, because the rest of the data is filtered by clinical data
    #Sample Clinical Data
    retract_samples(syn,process_functions.getDatabaseSynId(syn,"sample", test=test),"SAMPLE_ID",allRetractedSamples)
    #Patient Clinical Data
    retract_samples(syn, process_functions.getDatabaseSynId(syn, "patient", test=test),"PATIENT_ID",patientRetractIds['geniePatientId'])

def main():
    '''
    Main block with argparse and calls the main retract function
    '''
    parser = argparse.ArgumentParser(description='Sample retraction')
    parser.add_argument("--pemFile", type=str, help="Path to PEM file (genie.pem)")
    parser.add_argument("--test", action='store_true',help="Run test")
    parser.add_argument("--debug", action='store_true',help="Synapse Debug Feature")
    args = parser.parse_args()
    syn = process_functions.synLogin(args.pemFile, debug=args.debug)
    retract(syn, args.test)

if __name__ == "__main__":
    main()





import logging
import os

import pandas as pd
import synapseclient

from .example_filetype_format import FileTypeFormat
from . import process_functions

logger = logging.getLogger(__name__)


def validateSymbol(gene, bedDf, returnMappedDf=True):
    '''
    Validate gene symbol

    Args:
        gene: Gene name
        bedDf: Bed pandas dataframe
        returnMappedDf: Return a mapped gene. Defaults to True

    Returns:
        gene name or boolean for whether a gene is valid
    '''
    valid = False
    if sum(bedDf['Hugo_Symbol'] == gene) > 0:
        valid = True
    elif sum(bedDf['ID'] == gene) > 0:
        mismatch = bedDf[bedDf['ID'] == gene]
        mismatch.drop_duplicates(inplace=True)
        logger.info("{} will be remapped to {}".format(
            gene, mismatch['Hugo_Symbol'].values[0]))
        gene = mismatch['Hugo_Symbol'].values[0]
    else:
        logger.warning(
            "{} cannot be remapped and will not be released. The symbol "
            "must exist in your seq assay ids (bed files) and must be "
            "mappable to a gene.".format(gene))
        gene = float('nan')
    if returnMappedDf:
        return(gene)
    else:
        return(valid)


def makeCNARow(row, symbols):
    '''
    Make CNA Row (Deprecated function)

    CNA values are no longer stored in the database

    Args:
        row: one row in the CNA file
        symbols:  list of Gene symbols
    '''
    totalrow = "{symbols}\n{values}".format(
        symbols=",".join(symbols),
        values=",".join(row.astype(str)))
    totalrow = totalrow.replace(".0", "")
    return(totalrow)


def mergeCNAvalues(x):
    x.dropna(inplace=True)
    uniqueValues = set(x.unique())
    if len(uniqueValues) == 1:
        returnVal = x.tolist()[0]
    elif len(uniqueValues) <= 2:
        uniqueValues.discard(0)
        if len(uniqueValues) == 1:
            returnVal = list(uniqueValues)[0]
        else:
            returnVal = float('nan')
    else:
        returnVal = float('nan')
    return(returnVal)


def checkIfOneZero(x):
    assert len(set(x.tolist())) == 1, "Can only be one unique value"


class cna(FileTypeFormat):

    _fileType = "cna"

    _process_kwargs = ["newPath", 'databaseToSynIdMappingDf']

    _validation_kwargs = ['nosymbol_check']

    # VALIDATE FILENAME
    def _validateFilename(self, filePath):
        assert os.path.basename(filePath[0]) == \
            "data_CNA_{}.txt".format(self.center)

    def _process(self, cnaDf):
        cnaDf.rename(columns={
            cnaDf.columns[0]: cnaDf.columns[0].upper()}, inplace=True)
        cnaDf.rename(columns={
            "HUGO_SYMBOL": "Hugo_Symbol"}, inplace=True)

        index = [i for i, col in enumerate(cnaDf.columns)
                 if col.upper() == "ENTREZ_GENE_ID"]
        if len(index) > 0:
            del cnaDf[cnaDf.columns[index][0]]
        bedSynId = process_functions.getDatabaseSynId(
            self.syn, "bed", test=self.testing)
        bed = self.syn.tableQuery(
            "select Hugo_Symbol, ID from {} where CENTER = '{}'" .format(
                bedSynId, self.center))
        bedDf = bed.asDataFrame()
        cnaDf['Hugo_Symbol'] = \
            cnaDf['Hugo_Symbol'].apply(lambda x: validateSymbol(x, bedDf))
        order = cnaDf.columns
        cnaDf = cnaDf[~cnaDf['Hugo_Symbol'].isnull()]
        # cnaDf = cnaDf.applymap(str)
        duplicatedGenes = pd.DataFrame()
        duplicated_symbols = cnaDf['Hugo_Symbol'][
            cnaDf['Hugo_Symbol'].duplicated()].unique()
        for i in duplicated_symbols:
            dups = cnaDf[cnaDf['Hugo_Symbol'] == i]
            newVal = dups[dups.columns[dups.columns != "Hugo_Symbol"]].apply(
                mergeCNAvalues)
            temp = pd.DataFrame(newVal).transpose()
            temp['Hugo_Symbol'] = i
            duplicatedGenes = duplicatedGenes.append(temp, sort=False)
        cnaDf.drop_duplicates('Hugo_Symbol', keep=False, inplace=True)
        cnaDf = cnaDf.append(duplicatedGenes, sort=False)
        cnaDf = cnaDf[order]
        cnaDf.columns = [
            process_functions.checkGenieId(i, self.center)
            if i != "Hugo_Symbol" else i for i in cnaDf.columns]

        return(cnaDf)

    def process_steps(self, cnaDf, newPath, databaseToSynIdMappingDf):

        newCNA = self._process(cnaDf)

        centerMafSynId = databaseToSynIdMappingDf.Id[
            databaseToSynIdMappingDf['Database'] == "centerMaf"][0]
        if not newCNA.empty:
            cnaText = process_functions.removePandasDfFloat(newCNA)
            # Replace blank with NA's
            cnaText = cnaText.replace(
                "\t\t", "\tNA\t").replace(
                "\t\t", "\tNA\t").replace(
                '\t\n', "\tNA\n")
            with open(newPath, "w") as cnaFile:
                cnaFile.write(cnaText)
            self.syn.store(synapseclient.File(newPath, parent=centerMafSynId))
        return(newPath)

    def _validate(self, cnvDF, nosymbol_check):
        total_error = ""
        warning = ""
        cnvDF.columns = [col.upper() for col in cnvDF.columns]

        if cnvDF.columns[0] != "HUGO_SYMBOL":
            total_error += "Your cnv file's first column must be Hugo_Symbol\n"
        haveColumn = process_functions.checkColExist(cnvDF, "HUGO_SYMBOL")
        if haveColumn:
            keepSymbols = cnvDF["HUGO_SYMBOL"]
            cnvDF.drop("HUGO_SYMBOL", axis=1, inplace=True)

        # if sum(cnvDF.apply(lambda x: sum(x.isnull()))) > 0:
        #   total_error += "Your cnv file must not have any empty values\n"

        if process_functions.checkColExist(cnvDF, "ENTREZ_GENE_ID"):
            del cnvDF['ENTREZ_GENE_ID']

        # cnvDF = cnvDF.fillna('')
        allowed_values = ['-2.0', '-2', '-1.5', '-1.0', '-1',
                          '0.0', '0', '0.5', '1.0', '1', '1.5',
                          '2', '2.0', 'nan']
        if not all(cnvDF.applymap(lambda x: str(x) in allowed_values).all()):
            total_error += (
                "All values must be NA/blank, -2, -1.5, -1, -0.5, "
                "0, 0.5, 1, 1.5, or 2.\n")
        else:
            cnvDF['HUGO_SYMBOL'] = keepSymbols
            if haveColumn and not nosymbol_check:
                bedSynId = process_functions.getDatabaseSynId(
                    self.syn, "bed", test=self.testing)
                bed = self.syn.tableQuery(
                    "select Hugo_Symbol, ID from {} where "
                    "CENTER = '{}'".format(bedSynId, self.center))
                bedDf = bed.asDataFrame()
                cnvDF['remapped'] = cnvDF['HUGO_SYMBOL'].apply(
                    lambda x: validateSymbol(x, bedDf))
                cnvDF = cnvDF[~cnvDF['remapped'].isnull()]

                # Do not allow any duplicated genes after symbols
                # have been remapped
                if sum(cnvDF['remapped'].duplicated()) > 0:
                    duplicated = cnvDF['remapped'].duplicated(keep=False)
                    total_error += (
                        "Your CNA file has duplicated Hugo_Symbols "
                        "(After remapping of genes): {} -> {}.\n".format(
                            ",".join(cnvDF['HUGO_SYMBOL'][duplicated]),
                            ",".join(cnvDF['remapped'][duplicated])))
        return(total_error, warning)



"""Assay information class"""
import os
import yaml

import pandas as pd

from .example_filetype_format import FileTypeFormat
from . import process_functions


class Assayinfo(FileTypeFormat):
    """Assay information file type"""

    _fileType = "assayinfo"

    _process_kwargs = ["newPath", "databaseSynId"]

    def _validateFilename(self, filepath_list):
        """Validate assay information filename"""
        assert os.path.basename(filepath_list[0]) == "assay_information.yaml"

    def process_steps(self, assay_info_df, newPath, databaseSynId):
        """
        Process bed input and update bed database

        Args:
            assay_info_df: Assay information dataframe
            newPath: Path to processed assay information
            databaseSynId: assay information database synapse id

        Returns:
            path to assay information dataframe
        """
        # Must pass in a list
        process_assay_info_df = self._process(assay_info_df)
        process_functions.updateData(self.syn, databaseSynId,
                                     process_assay_info_df, self.center,
                                     filterByColumn="CENTER",
                                     toDelete=True)
        process_assay_info_df.to_csv(newPath, sep="\t", index=False)
        return newPath

    def _process(self, df):
        """
        Process assay_information.yaml. Standardizes SEQ_ASSAY_ID,
        default 10 for gene_padding, and fills in variant_classifications

        Args:
            df: Assay information dataframe

        Returns:
            dataframe: Processed dataframe
        """
        seq_assay_ids = [assay.upper().replace('_', '-')
                         for assay in df['SEQ_ASSAY_ID']]
        df['SEQ_ASSAY_ID'] = seq_assay_ids
        df['SEQ_PIPELINE_ID'] = [assay.upper().replace('_', '-')
                                 for assay in df['SEQ_PIPELINE_ID']]
        if process_functions.checkColExist(df, "gene_padding"):
            df['gene_padding'] = df['gene_padding'].fillna(10)
            df['gene_padding'] = df['gene_padding'].astype(int)
        else:
            df['gene_padding'] = 10

        if not process_functions.checkColExist(df, "variant_classifications"):
            df['variant_classifications'] = float('nan')

        df['CENTER'] = self.center
        return df

    def _get_dataframe(self, filepath_list):
        """Take in yaml file, returns dataframe"""
        filepath = filepath_list[0]
        try:
            with open(filepath, 'r') as yamlfile:
                # https://github.com/yaml/pyyaml/wiki/PyYAML-yaml.load(input)-Deprecation
                # Must add this because yaml load deprecation
                assay_info_dict = yaml.load(yamlfile, Loader=yaml.FullLoader)
        except Exception:
            raise ValueError(
                "assay_information.yaml: Can't read in your file. "
                "Please make sure the file is a correctly formatted yaml")
        # assay_info_df = pd.DataFrame(panel_info_dict)
        # assay_info_df = assay_info_df.transpose()
        # assay_info_df['SEQ_ASSAY_ID'] = assay_info_df.index
        # assay_info_df.reset_index(drop=True, inplace=True)
        assay_infodf = pd.DataFrame(assay_info_dict)
        assay_info_transposeddf = assay_infodf.transpose()

        all_panel_info = pd.DataFrame()
        for assay in assay_info_dict:
            assay_specific_info = assay_info_dict[assay]['assay_specific_info']
            assay_specific_infodf = pd.DataFrame(assay_specific_info)

            seq_assay_id_infodf = assay_info_transposeddf.loc[[assay]]

            to_appenddf = [seq_assay_id_infodf]*(len(assay_specific_info) - 1)
            if to_appenddf:
                seq_assay_id_infodf = seq_assay_id_infodf.append(to_appenddf)
            seq_assay_id_infodf.reset_index(drop=True, inplace=True)
            assay_finaldf = pd.concat(
                [assay_specific_infodf, seq_assay_id_infodf], axis=1)
            del assay_finaldf['assay_specific_info']

            columns_containing_lists = ['variant_classifications',
                                        'alteration_types',
                                        'preservation_technique', 'coverage']

            for col in columns_containing_lists:
                if assay_finaldf.get(col) is not None:
                    assay_finaldf[col] = [";".join(row)
                                          for row in assay_finaldf[col]]
            assay_finaldf['SEQ_PIPELINE_ID'] = assay
            all_panel_info = all_panel_info.append(assay_finaldf)
        return all_panel_info

    def _validate(self, assay_info_df):
        """
        Validates the values of assay information file

        Args:
            assay_info_df: assay information dataframe

        Returns:
            tuple: error and warning
        """

        total_error = ""
        warning = ""

        if process_functions.checkColExist(assay_info_df, "SEQ_ASSAY_ID"):
            all_seq_assays = assay_info_df.SEQ_ASSAY_ID.unique()
            if not all([assay.startswith(self.center)
                        for assay in all_seq_assays]):
                total_error += \
                    ("Assay_information.yaml: Please make sure your all your "
                     "SEQ_ASSAY_IDs start with your center abbreviation.\n")
        else:
            total_error += \
                "Assay_information.yaml: Must have SEQ_ASSAY_ID column.\n"

        read_group_dict = process_functions.get_gdc_data_dictionary(
            "read_group")
        read_group_headers = read_group_dict['properties']
        warn, error = process_functions.check_col_and_values(
            assay_info_df,
            'is_paired_end',
            [True, False],
            filename="Assay_information.yaml",
            required=True)
        warning += warn
        total_error += error

        warn, error = process_functions.check_col_and_values(
            assay_info_df, 'library_selection',
            read_group_headers['library_selection']['enum'],
            filename="Assay_information.yaml",
            required=True)
        warning += warn
        total_error += error

        warn, error = process_functions.check_col_and_values(
            assay_info_df,
            'library_strategy',
            read_group_headers['library_strategy']['enum'],
            filename="Assay_information.yaml",
            required=True)
        warning += warn
        total_error += error

        warn, error = process_functions.check_col_and_values(
            assay_info_df,
            'platform',
            read_group_headers['platform']['enum'],
            filename="Assay_information.yaml",
            required=True)
        warning += warn
        total_error += error

        instrument_model = read_group_headers['instrument_model']['enum']
        instrument_model.append(None)
        warn, error = process_functions.check_col_and_values(
            assay_info_df,
            'instrument_model',
            instrument_model,
            filename="Assay_information.yaml",
            required=True)
        warning += warn
        total_error += error

        # target_capture_kit = read_group_headers['target_capture_kit']['enum']
        # warn, error = process_functions.check_col_and_values(
        #     assay_info_df,
        #     'target_capture_kit',
        #     target_capture_kit,
        #     filename="Assay_information.yaml",
        #     required=True)
        # warning += warn
        # total_error += error

        if not process_functions.checkColExist(assay_info_df,
                                               "target_capture_kit"):
            total_error += ("Assay_information.yaml: "
                            "Must have target_capture_kit column.\n")

        variant_classes = ['Splice_Site', 'Nonsense_Mutation',
                           'Frame_Shift_Del', 'Frame_Shift_Ins',
                           'Nonstop_Mutation', 'Translation_Start_Site',
                           'In_Frame_Ins', 'In_Frame_Del',
                           'Missense_Mutation', 'Intron', 'Splice_Region',
                           'Silent', 'RNA', "5'UTR", "3'UTR", 'IGR',
                           "5'Flank", "3'Flank", None]
        warn, error = process_functions.check_col_and_values(
            assay_info_df,
            'variant_classifications',
            variant_classes,
            filename="Assay_information.yaml",
            na_allowed=True,
            sep=";")
        warning += warn
        total_error += error

        if process_functions.checkColExist(assay_info_df, "read_length"):
            if not all([process_functions.checkInt(i)
                        for i in assay_info_df["read_length"]
                        if i is not None and not pd.isnull(i)]):
                total_error += \
                    ("Assay_information.yaml: "
                     "Please double check your read_length.  "
                     "It must be an integer or null.\n")
        else:
            total_error += \
                ("Assay_information.yaml: "
                 "Must have read_length column.\n")

        if process_functions.checkColExist(assay_info_df, "number_of_genes"):
            if not all([process_functions.checkInt(i)
                        for i in assay_info_df["number_of_genes"]]):
                total_error += \
                    ("Assay_information.yaml: "
                     "Please double check your number_of_genes. "
                     "It must be an integer.\n")
        else:
            total_error += \
                ("Assay_information.yaml: "
                 "Must have number_of_genes column.\n")

        if process_functions.checkColExist(assay_info_df, "gene_padding"):
            if not all([process_functions.checkInt(i)
                        for i in assay_info_df["gene_padding"]
                        if i is not None and not pd.isnull(i)]):
                total_error += \
                    ("Assay_information.yaml: "
                     "Please double check your gene_padding. "
                     "It must be an integer or blank.\n")
        else:
            warning += \
                ("Assay_information.yaml: "
                 "gene_padding is by default 10 if not specified.\n")

        warn, error = process_functions.check_col_and_values(
            assay_info_df, 'calling_strategy', ['tumor_only', 'tumor_normal'],
            filename="Assay_information.yaml", required=True)
        warning += warn
        total_error += error

        if process_functions.checkColExist(assay_info_df, "specimen_tumor_cellularity"):
            if not all([i.startswith(">") and i.endswith("%")
                        for i in assay_info_df["specimen_tumor_cellularity"]]):
                total_error += \
                    ("Assay_information.yaml: "
                     "Please double check your specimen_tumor_cellularity. "
                     "It must in this format >(num)%. ie. >10%\n")
        else:
            total_error += \
                ("Assay_information.yaml: "
                 "Must have specimen_tumor_cellularity column.\n")

        alteration_types = ["snv", "small_indels", "gene_level_cna",
                            "intragenic_cna", "structural_variants"]
        warn, error = process_functions.check_col_and_values(
            assay_info_df, 'alteration_types', alteration_types,
            filename="Assay_information.yaml", required=True, sep=";")
        warning += warn
        total_error += error

        preservation_technique = ["FFPE", 'fresh_frozen']
        warn, error = process_functions.check_col_and_values(
            assay_info_df, 'preservation_technique', preservation_technique,
            filename="Assay_information.yaml", required=True, sep=";")
        warning += warn
        total_error += error

        coverage = ['hotspot_regions', 'coding_exons', 'introns', 'promoters']
        warn, error = process_functions.check_col_and_values(
            assay_info_df, 'coverage', coverage,
            filename="Assay_information.yaml", required=True, sep=";")
        warning += warn
        total_error += error

        return total_error, warning



#!/usr/bin/env python3
from collections import defaultdict
import datetime
import logging
import os
import time

import synapseclient
try:
    from synapseclient.core.utils import to_unix_epoch_time
except ModuleNotFoundError:
    from synapseclient.utils import to_unix_epoch_time
import synapseutils
import pandas as pd

from .config import PROCESS_FILES
from . import process_functions
from . import validate
from . import toRetract

logging.basicConfig(level=logging.INFO)
logger = logging.getLogger(__name__)

'''
TODO:
Could potentially get all the inforamation of the file entity right here
To avoid the syn.get rest call later which doesn't actually download the file
'''

# def rename_file(ent):
#     '''
#     Gets file from synapse and renames the file if necessary.

#     Adds the expected name as an annotation to a Synapse File object.

#     Args:
#         synid : Synapse id or entity

#     Returns:
#         entity with annotation set for path of corrected file
#     '''
#     dirpath = os.path.dirname(ent.path)
#     expectedpath = os.path.join(dirpath, ent.name)

#     ent.annotations.expectedPath = expectedpath
#     return ent


def entity_date_to_timestamp(entity_date_time):
    """Convert Synapse object date/time string (from modifiedOn or createdOn properties) to a timestamp.
    """

    date_and_time = entity_date_time.split(".")[0]
    date_time_obj = datetime.datetime.strptime(date_and_time, "%Y-%m-%dT%H:%M:%S")
    return to_unix_epoch_time(date_time_obj)


def get_center_input_files(syn, synid, center, process="main"):
    '''
    This function walks through each center's input directory
    to get a list of tuples of center files

    Args:
        syn: Synapse object
        synid: Center input folder synid
        center: Center name
        process: Process type includes, main, vcf, maf and mafSP.
                 Defaults to main such that the vcf

    Returns:
        List of entities with the correct format to pass into validation
    '''
    logger.info("GETTING {center} INPUT FILES".format(center=center))
    clinical_pair_name = [
        "data_clinical_supp_sample_{center}.txt".format(center=center),
        "data_clinical_supp_patient_{center}.txt".format(center=center)]

    center_files = synapseutils.walk(syn, synid)
    clinicalpair_entities = []
    prepared_center_file_list = []

    for _, _, entities in center_files:
        for name, ent_synid in entities:
            # This is to remove vcfs from being validated during main
            # processing. Often there are too many vcf files, and it is
            # not necessary for them to be run everytime.
            if name.endswith(".vcf") and process != "vcf":
                continue

            ent = syn.get(ent_synid)
            logger.debug(ent)

            # Clinical file can come as two files.
            # The two files need to be merged together which is
            # why there is this format

            if name in clinical_pair_name:
                clinicalpair_entities.append(ent)
                continue

            prepared_center_file_list.append([ent])

    if clinicalpair_entities:
        # clinicalpair_entities = [x for x in clinicalpair]
        prepared_center_file_list.append(clinicalpair_entities)

    return prepared_center_file_list


def check_existing_file_status(validation_status_table, error_tracker_table, entities):
    '''
    This function checks input files against the existing validation and error
    tracking dataframe

    Args:
        validation_status_table: Validation status Synapse Table query result
        error_tracker_table: Error tracking Synapse Table query result
        entities: list of center input entites

    Returns:
        dict: Input file status
            status_list: file validation status
            error_list: Errors of the files if they exist,
            to_validate: Boolean value for whether of not an input
                         file needs to be validated
    '''
    if len(entities) > 2:
        raise ValueError(
            "There should never be more than 2 files being validated.")

    statuses = []
    errors = []

    validation_statusdf = validation_status_table.asDataFrame()
    error_trackerdf = error_tracker_table.asDataFrame()
    # This should be outside fo the forloop so that it doesn't
    # get reset
    to_validate = False
    for ent in entities:
        # Get the current status and errors from the tables.
        current_status = validation_statusdf[validation_statusdf['id'] == ent.id]
        current_error = error_trackerdf[error_trackerdf['id'] == ent.id]

        if current_status.empty:
            to_validate = True
        else:
            # This to_validate is here, because the following is a
            # sequential check of whether files need to be validated
            statuses.append(current_status['status'].values[0])
            if current_error.empty:
                to_validate = \
                    current_status['status'].values[0] == "INVALID"
            else:
                errors.append(current_error['errors'].values[0])
            # Add Name check here (must add name of the entity as a column)
            if current_status['md5'].values[0] != ent.md5 or \
               current_status['name'].values[0] != ent.name:
                to_validate = True
            else:
                status_str = "{filename} ({id}) FILE STATUS IS: {filestatus}"
                logger.info(status_str.format(filename=ent.name, id=ent.id,
                                              filestatus=current_status['status'].values[0]))

    return({'status_list': statuses,
            'error_list': errors,
            'to_validate': to_validate})


def _send_validation_error_email(syn, user, message_objs):
    '''
    Sends validation error email

    Args:
        syn: Synapse object
        user: username to send message to
        message_objs: list of dicts with 'filenames' and 'messages' to send
    '''

    username = syn.getUserProfile(user)['userName']

    errors = ""
    for message_obj in message_objs:
        file_names = ", ".join(message_obj['filenames'])
        error_message = message_obj['messages']
        errors += f"Filenames: {file_names}, Errors:\n {error_message}\n\n"

    email_message = (f"Dear {username},\n\n"
                     "You have invalid files! "
                     f"Here are the reasons why:\n\n{errors}")

    date_now = datetime.datetime.today().strftime("%Y-%m-%d %H:%M:%S")

    syn.sendMessage(userIds=[user],
                    messageSubject=f"GENIE Validation Error - {date_now}",
                    messageBody=email_message)


def _get_status_and_error_list(valid, message, entities):
    '''
    Helper function to return the status and error list of the
    files based on validation result.

    Args:
        valid: Boolean value of results of validation
        message: Validation message
        entities: List of Synapse Entities

    Returns:
        tuple: input_status_list - status of input files list,
               invalid_errors_list - error list
    '''
    if valid:
        input_status_list = [{'entity': ent, 'status': "VALIDATED"}
                             for ent in entities]
        invalid_errors_list = []
    else:
        input_status_list = [{'entity': ent, 'status': "INVALID"}
                             for ent in entities]
        invalid_errors_list = [{'entity': ent, 'errors': message}
                               for ent in entities]
    return input_status_list, invalid_errors_list


def validatefile(syn, entities, validation_status_table, error_tracker_table,
                 center, testing, oncotree_link,
                 format_registry=PROCESS_FILES):
    '''Validate a list of entities.

    If a file has not changed, then it doesn't need to be validated.

    Args:
        syn: Synapse object
        entities: A list of entities for a single file 'type' (usually a single file, but clinical can have two)
        validation_statusdf: Validation status dataframe
        error_trackerdf: Invalid files error tracking dataframe
        center: Center of interest
        testing: Boolean determining whether using testing parameter
        oncotree_link: Oncotree url

    Returns:
        tuple: input_status_list - status of input files,
               invalid_errors_list - error list
               messages_to_send - list of tuples with (filenames, message, file_users)

    '''

    filepaths = [entity.path for entity in entities]
    filenames = [entity.name for entity in entities]

    logger.info("VALIDATING {filenames}".format(filenames=", ".join(filenames)))

    file_users = [entities[0].modifiedBy, entities[0].createdBy]

    check_file_status = check_existing_file_status(
        validation_status_table, error_tracker_table, entities)

    status_list = check_file_status['status_list']
    error_list = check_file_status['error_list']

    messages_to_send = []

    # Need to figure out to how to remove this
    # This must pass in filenames, because filetype is determined by entity
    # name Not by actual path of file
    validator = validate.GenieValidationHelper(syn=syn, center=center,
                                               entitylist=entities,
                                               format_registry=format_registry,
                                               testing=testing)
    filetype = validator.file_type
    if check_file_status['to_validate']:
        valid, message = validator.validate_single_file(
            oncotree_link=oncotree_link, nosymbol_check=False)
        logger.info("VALIDATION COMPLETE")
        input_status_list, invalid_errors_list = _get_status_and_error_list(
            valid, message, entities)
        # Send email the first time the file is invalid
        if invalid_errors_list:
            messages_to_send.append((filenames, message, file_users))
    else:
        input_status_list = [{'entity': entity, 'status': status}
                             for entity, status in zip(entities, status_list)]
        invalid_errors_list = [{'entity': entity, 'errors': errors}
                               for entity, errors in zip(entities, error_list)]
    # add in static filetype and center information
    for input_status in input_status_list:
        input_status.update({'fileType': filetype, 'center': center})
    # An empty list is returned if there are no errors,
    # so nothing will be appended
    for invalid_errors in invalid_errors_list:
        invalid_errors.update({'fileType': filetype, 'center': center})
    return input_status_list, invalid_errors_list, messages_to_send


def processfiles(syn, validfiles, center, path_to_genie,
                 center_mapping_df, oncotree_link, databaseToSynIdMappingDf,
                 validVCF=None, vcf2mafPath=None,
                 veppath=None, vepdata=None,
                 processing="main", test=False, reference=None):
    '''
    Processing validated files

    Args:
        syn: Synapse object
        validfiles: pandas dataframe containing validated files
                    has 'id', 'path', and 'fileType' column
        center: GENIE center name
        path_to_genie: Path to GENIE workdir
        center_mapping_df: Center mapping dataframe
        oncotree_link: Link to oncotree
        databaseToSynIdMappingDf: Database to synapse id mapping dataframe
        validVCF: Valid vcf files
        vcf2mafPath: Path to vcf2maf
        veppath: Path to vep
        vepdata: Path to vep index files
        processing: Processing type. Defaults to main
        test: Test flag
        reference: Reference file for vcf2maf
    '''
    logger.info("PROCESSING {} FILES: {}".format(center, len(validfiles)))
    center_staging_folder = os.path.join(path_to_genie, center)
    center_staging_synid = center_mapping_df.query(
        "center == '{}'".format(center)).stagingSynId.iloc[0]

    if not os.path.exists(center_staging_folder):
        os.makedirs(center_staging_folder)

    if processing != 'vcf':
        for fileSynId, filePath, name, fileType in zip(validfiles['id'],
                                                       validfiles['path'],
                                                       validfiles['name'],
                                                       validfiles['fileType']):
            # filename = os.path.basename(filePath)
            newPath = os.path.join(center_staging_folder, name)
            # store = True
            synId = databaseToSynIdMappingDf.Id[
                databaseToSynIdMappingDf['Database'] == fileType]
            if len(synId) == 0:
                synId = None
            else:
                synId = synId[0]
            if fileType is not None and (processing == "main" or processing == fileType):
                processor = PROCESS_FILES[fileType](syn, center)
                processor.process(
                    filePath=filePath, newPath=newPath,
                    parentId=center_staging_synid, databaseSynId=synId,
                    oncotree_link=oncotree_link,
                    fileSynId=fileSynId, validVCF=validVCF,
                    path_to_GENIE=path_to_genie, vcf2mafPath=vcf2mafPath,
                    veppath=veppath, vepdata=vepdata,
                    processing=processing,
                    databaseToSynIdMappingDf=databaseToSynIdMappingDf,
                    reference=reference, test=test)

    else:
        filePath = None
        newPath = None
        fileType = None
        synId = databaseToSynIdMappingDf.Id[
            databaseToSynIdMappingDf['Database'] == processing][0]
        fileSynId = None
        processor = PROCESS_FILES[processing](syn, center)
        processor.process(
            filePath=filePath, newPath=newPath,
            parentId=center_staging_synid, databaseSynId=synId,
            oncotree_link=oncotree_link,
            fileSynId=fileSynId, validVCF=validVCF,
            path_to_GENIE=path_to_genie, vcf2mafPath=vcf2mafPath,
            veppath=veppath, vepdata=vepdata,
            processing=processing,
            databaseToSynIdMappingDf=databaseToSynIdMappingDf,
            reference=reference)

    logger.info("ALL DATA STORED IN DATABASE")

# def _create_maf_db(syn, foo):
#     maf_database_ent = syn.get(maf_database_synid)
#     print(maf_database_ent)
#     maf_columns = list(syn.getTableColumns(maf_database_synid))
#     schema = synapseclient.Schema(
#         name='Narrow MAF {current_time} Database'.format(
#             current_time=time.time()),
#         columns=maf_columns,
#         parent=process_functions.getDatabaseSynId(
#             syn, "main",
#             databaseToSynIdMappingDf=database_synid_mappingdf))
#     schema.primaryKey = maf_database_ent.primaryKey
#     new_maf_database = syn.store(schema)

# TODO: Should split this into 3 funcitons
# so that unit tests are easier to write


def create_and_archive_maf_database(syn, database_synid_mappingdf):
    '''
    Creates new MAF database and archives the old database in the staging site

    Args:
        syn: Synapse object
        databaseToSynIdMappingDf: Database to synapse id mapping dataframe

    Return:
        Editted database to synapse id mapping dataframe
    '''
    maf_database_synid = process_functions.getDatabaseSynId(
        syn, "vcf2maf", databaseToSynIdMappingDf=database_synid_mappingdf)
    maf_database_ent = syn.get(maf_database_synid)
    maf_columns = list(syn.getTableColumns(maf_database_synid))
    schema = synapseclient.Schema(
        name='Narrow MAF {current_time} Database'.format(
            current_time=time.time()),
        columns=maf_columns,
        parent=process_functions.getDatabaseSynId(
            syn, "main", databaseToSynIdMappingDf=database_synid_mappingdf))
    schema.primaryKey = maf_database_ent.primaryKey
    new_maf_database = syn.store(schema)
    # Store in the new database synid
    database_synid_mappingdf['Id'][
        database_synid_mappingdf[
            'Database'] == 'vcf2maf'] = new_maf_database.id

    vcf2maf_mappingdf = database_synid_mappingdf[
        database_synid_mappingdf['Database'] == 'vcf2maf']
    # vcf2maf_mappingdf['Id'][0] = newMafDb.id
    # Update this synid later
    syn.store(synapseclient.Table("syn12094210", vcf2maf_mappingdf))
    # Move and archive old mafdatabase (This is the staging synid)
    maf_database_ent.parentId = "syn7208886"
    maf_database_ent.name = "ARCHIVED " + maf_database_ent.name
    syn.store(maf_database_ent)
    # maf_database_synid = new_maf_database.id
    # Remove can download permissions from project GENIE team
    syn.setPermissions(new_maf_database.id, 3326313, [])
    return(database_synid_mappingdf)


def email_duplication_error(syn, duplicated_filesdf):
    '''
    Sends an email if there is a duplication error

    Args:
        syn: Synapse object
        duplicated_filesdf: dataframe with 'id', 'name' column
    '''
    if not duplicated_filesdf.empty:
        incorrect_files = [
            name for synId, name in zip(duplicated_filesdf['id'],
                                        duplicated_filesdf['name'])]
        incorrect_filenames = ", ".join(incorrect_files)
        incorrect_ent = syn.get(duplicated_filesdf['id'].iloc[0])
        send_to_users = set([incorrect_ent.modifiedBy,
                             incorrect_ent.createdBy])
        usernames = ", ".join(
            [syn.getUserProfile(user)['userName'] for user in send_to_users])
        error_email = (
            "Dear {},\n\n"
            "Your files ({}) are duplicated!  FILES SHOULD BE UPLOADED AS "
            "NEW VERSIONS AND THE ENTIRE DATASET SHOULD BE "
            "UPLOADED EVERYTIME".format(usernames, incorrect_filenames))
        syn.sendMessage(
            list(send_to_users), "GENIE Validation Error", error_email)


def get_duplicated_files(validation_statusdf, duplicated_error_message):
    '''
    Check for duplicated files.  There should be no duplication,
    files should be uploaded as new versions and the entire dataset
    should be uploaded everytime

    Args:
        validation_statusdf: dataframe with 'name' and 'id' column
        duplicated_error_message: Error message for duplicated files

    Returns:
        dataframe with 'id', 'name', 'errors', 'center', 'fileType' of
        duplicated files
    '''
    logger.info("CHECK FOR DUPLICATED FILES")
    duplicated_filesdf = validation_statusdf[
        validation_statusdf['name'].duplicated(keep=False)]
    # Define filename str vector
    filename_str = validation_statusdf.name.str
    # cbs/seg files should not be duplicated.
    cbs_seg_index = filename_str.endswith(("cbs", "seg"))
    cbs_seg_files = validation_statusdf[cbs_seg_index]
    if len(cbs_seg_files) > 1:
        duplicated_filesdf = duplicated_filesdf.append(cbs_seg_files)
    # clinical files should not be duplicated.
    clinical_index = filename_str.startswith("data_clinical_supp")
    clinical_files = validation_statusdf[clinical_index]
    if len(clinical_files) > 2:
        duplicated_filesdf = duplicated_filesdf.append(clinical_files)
    duplicated_filesdf.drop_duplicates("id", inplace=True)
    logger.info("THERE ARE {} DUPLICATED FILES".format(
        len(duplicated_filesdf)))
    duplicated_filesdf['errors'] = duplicated_error_message
    return(duplicated_filesdf)


def update_status_and_error_tables(syn,
                                   input_valid_statuses,
                                   invalid_errors,
                                   validation_status_table,
                                   error_tracker_table):
    '''
    Update validation status and error tracking table

    Args:
        syn: Synapse object
        center: Center
        input_valid_status: list of lists of validation status
        invalid_errors: List of lists of invalid errors
        validation_status_table: Synapse table query of validation status
        error_tracker_table: Synapse table query of error tracker

    Returns:
        input validation status dataframe
    '''

    error_table_columns = ["id", 'errors', 'name', 'fileType', 'center']
    status_table_columns = ["id", 'path', 'md5', 'status', 'name',
                            'modifiedOn', 'fileType', 'center']

    input_status_rows = []
    for input_status in input_valid_statuses:
        entity = input_status['entity']
        row = {'id': entity.id,
               'path': entity.path,
               'md5': entity.md5,
               'status': input_status['status'],
               'name': entity.name,
               'modifiedOn': entity_date_to_timestamp(entity.properties.modifiedOn),
               'fileType': input_status['fileType'],
               'center': input_status['center']}
        input_status_rows.append(row)

    invalid_error_rows = []
    for invalid_error in invalid_errors:
        entity = invalid_error['entity']
        row = {'id': entity.id,
               'errors': invalid_error['errors'],
               'name': entity.name,
               'fileType': invalid_error['fileType'],
               'center': invalid_error['center']}
        invalid_error_rows.append(row)
    if input_status_rows:
        input_valid_statusdf = pd.DataFrame(input_status_rows)
    else:
        input_valid_statusdf = pd.DataFrame(input_status_rows,
                                            columns=status_table_columns)
    duplicated_file_error = (
        "DUPLICATED FILENAME! FILES SHOULD BE UPLOADED AS NEW VERSIONS "
        "AND THE ENTIRE DATASET SHOULD BE UPLOADED EVERYTIME")
    duplicated_filesdf = get_duplicated_files(input_valid_statusdf,
                                              duplicated_file_error)
    # Send an email if there are any duplicated files
    if not duplicated_filesdf.empty:
        email_duplication_error(syn, duplicated_filesdf)
    duplicated_idx = input_valid_statusdf['id'].isin(duplicated_filesdf['id'])
    input_valid_statusdf['status'][duplicated_idx] = "INVALID"
    # Create invalid error synapse table
    logger.info("UPDATE INVALID FILE REASON DATABASE")
    if invalid_error_rows:
        invalid_errorsdf = pd.DataFrame(invalid_error_rows)
    else:
        invalid_errorsdf = pd.DataFrame(invalid_error_rows,
                                        columns=error_table_columns)
    # Remove fixed duplicated files
    # This makes sure that the files removed actually had duplicated file
    # errors and not some other error
    dup_ids = invalid_errorsdf['id'][
        invalid_errorsdf['errors'] == duplicated_file_error]
    remove_ids = dup_ids[~dup_ids.isin(duplicated_filesdf['id'])]
    invalid_errorsdf = invalid_errorsdf[~invalid_errorsdf['id'].isin(remove_ids)]
    # Append duplicated file errors
    invalid_errorsdf = invalid_errorsdf.append(
        duplicated_filesdf[invalid_errorsdf.columns])
    invalid_ids = input_valid_statusdf['id'][input_valid_statusdf['status'] == "INVALID"]
    invalid_errorsdf = invalid_errorsdf[invalid_errorsdf['id'].isin(invalid_ids)]
    process_functions.updateDatabase(syn, error_tracker_table.asDataFrame(),
                                     invalid_errorsdf,
                                     error_tracker_table.tableId,
                                     ["id"], to_delete=True)

    logger.info("UPDATE VALIDATION STATUS DATABASE")
    # Remove fixed duplicated files
    input_valid_statusdf = input_valid_statusdf[
        ~input_valid_statusdf['id'].isin(remove_ids)]

    process_functions.updateDatabase(syn,
                                     validation_status_table.asDataFrame(),
                                     input_valid_statusdf,
                                     validation_status_table.tableId,
                                     ["id"],
                                     to_delete=True)

    return(input_valid_statusdf)


def validation(syn, center, process,
               center_mapping_df, database_synid_mappingdf,
               testing, oncotree_link, format_registry):
    '''
    Validation of all center files

    Args:
        syn: Synapse object
        center: Center name
        process: main, vcf, maf
        center_mapping_df: center mapping dataframe
        testing: True if testing
        oncotree_link: Link to oncotree

    Returns:
        dataframe: Valid files
    '''
    center_input_synid = center_mapping_df['inputSynId'][
        center_mapping_df['center'] == center][0]
    logger.info("Center: " + center)
    center_files = get_center_input_files(syn, center_input_synid, center,
                                          process)

    # If a center has no files, then return empty list
    if not center_files:
        logger.info("{} has not uploaded any files".format(center))
        return []
    else:
        logger.info(f"{center} has uploaded {len(center_files)} files.")

        validation_status_synid = process_functions.getDatabaseSynId(
            syn, "validationStatus",
            databaseToSynIdMappingDf=database_synid_mappingdf)
        error_tracker_synid = process_functions.getDatabaseSynId(
            syn, "errorTracker",
            databaseToSynIdMappingDf=database_synid_mappingdf)

        # Make sure the vcf validation statuses don't get wiped away
        # If process is not vcf, the vcf files are not downloaded
        add_query_str = "and name not like '%.vcf'" if process != "vcf" else ''

        # id, md5, status, name, center, modifiedOn, fileType
        validation_status_table = syn.tableQuery(
            "SELECT * FROM {synid} "
            "where center = '{center}' {add}".format(
                synid=validation_status_synid,
                center=center,
                add=add_query_str))
        # id, center, errors, name, fileType
        error_tracker_table = syn.tableQuery(
            "SELECT * FROM {synid} "
            "where center = '{center}' {add}".format(
                synid=error_tracker_synid,
                center=center,
                add=add_query_str))

        input_valid_statuses = []
        invalid_errors = []

        user_message_dict = defaultdict(list)

        for ents in center_files:
            status, errors, messages_to_send = validatefile(
                syn, ents,
                validation_status_table,
                error_tracker_table,
                center=center,
                testing=testing,
                oncotree_link=oncotree_link,
                format_registry=format_registry)

            input_valid_statuses.extend(status)
            if errors is not None:
                invalid_errors.extend(errors)

            if messages_to_send:
                logger.debug("Collating messages to send to users.")
                for filenames, messages, users in messages_to_send:
                    file_messages = dict(filenames=filenames,
                                         messages=messages)
                    # Must get unique set of users or there
                    # will be duplicated error messages sent in the email
                    for user in set(users):
                        user_message_dict[user].append(file_messages)

        for user, message_objs in user_message_dict.items():
            logger.debug("Sending messages to user {user}.".format(user=user))

            _send_validation_error_email(syn=syn,
                                         user=user,
                                         message_objs=message_objs)

        input_valid_statusdf = update_status_and_error_tables(
            syn,
            input_valid_statuses,
            invalid_errors,
            validation_status_table,
            error_tracker_table)

        valid_filesdf = input_valid_statusdf.query('status == "VALIDATED"')

        return(valid_filesdf[['id', 'path', 'fileType', 'name']])


def center_input_to_database(
        syn, center, process, testing,
        only_validate, vcf2maf_path, vep_path,
        vep_data, database_to_synid_mappingdf,
        center_mapping_df, reference=None,
        delete_old=False, oncotree_link=None):
    if only_validate:
        log_path = os.path.join(
            process_functions.SCRIPT_DIR,
            "{}_validation_log.txt".format(center))
    else:
        log_path = os.path.join(
            process_functions.SCRIPT_DIR,
            "{}_{}_log.txt".format(center, process))

    logFormatter = logging.Formatter(
        "%(asctime)s [%(name)s][%(levelname)s] %(message)s")
    fileHandler = logging.FileHandler(log_path, mode='w')
    fileHandler.setFormatter(logFormatter)
    logger.addHandler(fileHandler)

    if testing:
        logger.info("###########################################")
        logger.info("############NOW IN TESTING MODE############")
        logger.info("###########################################")

    # ----------------------------------------
    # Start input to staging process
    # ----------------------------------------
    '''
    # path_to_genie = os.path.realpath(os.path.join(
    #    process_functions.SCRIPT_DIR, "../"))
    Make the synapsecache dir the genie input folder for now
    The main reason for this is because the .synaspecache dir
    is mounted by batch
    '''
    path_to_genie = os.path.expanduser("~/.synapseCache")
    # Create input and staging folders
    if not os.path.exists(os.path.join(path_to_genie, center, "input")):
        os.makedirs(os.path.join(path_to_genie, center, "input"))
    if not os.path.exists(os.path.join(path_to_genie, center, "staging")):
        os.makedirs(os.path.join(path_to_genie, center, "staging"))

    if delete_old:
        process_functions.rmFiles(os.path.join(path_to_genie, center))

    validFiles = validation(
        syn, center, process, center_mapping_df,
        database_to_synid_mappingdf,
        testing, oncotree_link, PROCESS_FILES)

    if len(validFiles) > 0 and not only_validate:
        # Reorganize so BED file are always validated and processed first
        bed_files = validFiles['fileType'] == "bed"
        beds = validFiles[bed_files]
        validFiles = beds.append(validFiles)
        validFiles.drop_duplicates(inplace=True)
        # Valid vcf files
        vcf_files = validFiles['fileType'] == "vcf"
        validVCF = validFiles['path'][vcf_files].tolist()

        # merge clinical files into one row
        clinical_ind = validFiles['fileType'] == "clinical"
        if clinical_ind.any():
            clinical_files = validFiles[clinical_ind].to_dict(orient='list')
            # The [] implies the values in the dict as a list
            merged_clinical = pd.DataFrame([clinical_files])
            merged_clinical['fileType'] = 'clinical'
            merged_clinical['name'] = f"data_clinical_supp_{center}.txt"
            validFiles = validFiles[~clinical_ind].append(merged_clinical)

        processTrackerSynId = process_functions.getDatabaseSynId(
            syn, "processTracker",
            databaseToSynIdMappingDf=database_to_synid_mappingdf)
        # Add process tracker for time start
        processTracker = syn.tableQuery(
            "SELECT timeStartProcessing FROM {} "
            "where center = '{}' and "
            "processingType = '{}'".format(
                processTrackerSynId, center, process))
        processTrackerDf = processTracker.asDataFrame()
        if len(processTrackerDf) == 0:
            new_rows = [[
                center,
                str(int(time.time()*1000)),
                str(int(time.time()*1000)),
                process]]

            syn.store(synapseclient.Table(
                processTrackerSynId, new_rows))
        else:
            processTrackerDf['timeStartProcessing'][0] = \
                str(int(time.time()*1000))
            syn.store(synapseclient.Table(
                processTrackerSynId, processTrackerDf))

        processfiles(syn, validFiles, center, path_to_genie,
                     center_mapping_df, oncotree_link,
                     database_to_synid_mappingdf,
                     validVCF=validVCF,
                     vcf2mafPath=vcf2maf_path,
                     veppath=vep_path, vepdata=vep_data,
                     test=testing, processing=process, reference=reference)

        # Should add in this process end tracking
        # before the deletion of samples
        processTracker = syn.tableQuery(
            "SELECT timeEndProcessing FROM {synid} where center = '{center}' "
            "and processingType = '{processtype}'".format(
                synid=processTrackerSynId,
                center=center,
                processtype=process))
        processTrackerDf = processTracker.asDataFrame()
        processTrackerDf['timeEndProcessing'][0] = str(int(time.time()*1000))
        syn.store(synapseclient.Table(processTrackerSynId, processTrackerDf))

        logger.info("SAMPLE/PATIENT RETRACTION")
        toRetract.retract(syn, testing)

    else:
        messageOut = \
            "{} does not have any valid files" if not only_validate \
            else "ONLY VALIDATION OCCURED FOR {}"
        logger.info(messageOut.format(center))

    # Store log file
    log_folder_synid = process_functions.getDatabaseSynId(
        syn, "logs", databaseToSynIdMappingDf=database_to_synid_mappingdf)
    syn.store(synapseclient.File(log_path, parentId=log_folder_synid))
    os.remove(log_path)
    logger.info("ALL PROCESSES COMPLETE")



import datetime
import logging
import os

import pandas as pd
import synapseclient
try:
    from synapseclient.core.utils import to_unix_epoch_time
except ModuleNotFoundError:
    from synapseclient.utils import to_unix_epoch_time

from .example_filetype_format import FileTypeFormat
from . import process_functions

logger = logging.getLogger(__name__)


class sampleRetraction(FileTypeFormat):

    _fileType = "sampleRetraction"

    _process_kwargs = ["newPath", "databaseSynId","fileSynId"]

    def _get_dataframe(self, filePathList):
        '''
        This function by defaults assumes the filePathList is length of 1 
        and is a tsv file.  Could change depending on file type.
        '''
        filePath = filePathList[0]
        df = pd.read_csv(filePath,header=None)
        return(df)

    def _validateFilename(self, filePath):
        assert os.path.basename(filePath[0]) == "%s.csv" % self._fileType

    def _process(self, deleteSamplesDf, modifiedOn):
        col = 'genieSampleId' if self._fileType == "sampleRetraction" else 'geniePatientId'
        deleteSamplesDf.rename(columns = {0:col}, inplace=True)
        samples = [process_functions.checkGenieId(sample, self.center) for sample in deleteSamplesDf[col]]
        deleteSamplesDf[col] = samples
        modifiedOn = to_unix_epoch_time(datetime.datetime.strptime(modifiedOn, "%Y-%m-%dT%H:%M:%S"))
        deleteSamplesDf['retractionDate'] = modifiedOn
        deleteSamplesDf['center'] = self.center
        return(deleteSamplesDf)

    def process_steps(self, deleteSamples, fileSynId, databaseSynId, newPath):
        info = self.syn.get(fileSynId, downloadFile=False)
        deleteSamples = self._process(deleteSamples, info.modifiedOn.split(".")[0])
        process_functions.updateData(self.syn, databaseSynId, deleteSamples, self.center, filterByColumn="center", toDelete=True)
        return(newPath)



import logging
import os
import subprocess

import pandas as pd

from .maf import maf
from . import process_functions

logger = logging.getLogger(__name__)


def contains_whitespace(row):
    """Gets the total number of whitespaces from each column of a row"""
    return sum([" " in i for i in row if isinstance(i, str)])


class vcf(maf):
    _fileType = "vcf"

    _process_kwargs = ["validVCF", "processing", "path_to_GENIE",
                       "databaseToSynIdMappingDf", "vcf2mafPath",
                       "veppath", "vepdata", "reference"]

    def _validateFilename(self, filePath):
        basename = os.path.basename(filePath[0])
        startswith_genie = basename.startswith("GENIE-{}-".format(self.center))
        endswith_vcf = basename.endswith(".vcf")
        assert startswith_genie and endswith_vcf

    def _get_dataframe(self, filePathList):
        headers = None
        filepath = filePathList[0]
        with open(filepath, "r") as vcffile:
            for row in vcffile:
                if row.startswith("#CHROM"):
                    headers = \
                        row.replace("\n", "").replace("\r", "").split("\t")
        if headers is not None:
            vcfdf = pd.read_csv(filepath, sep="\t", comment="#", header=None,
                                names=headers)
        else:
            raise ValueError("Your vcf must start with the header #CHROM")
        return vcfdf

    def process_helper(self, vcffiles, path_to_GENIE, mafSynId, centerMafSynId,
                       vcf2mafPath, veppath, vepdata,
                       reference=None):
        logger.info('VCF2MAF %s' % self.center)
        centerInputFolder = os.path.join(path_to_GENIE, self.center, "input")
        centerStagingFolder = os.path.join(path_to_GENIE, self.center,
                                           "staging")
        mafFiles = []
        for path in vcffiles:
            vcfName = os.path.basename(path)
            logger.info(vcfName)
            newVCFPath = os.path.join(centerInputFolder, vcfName)
            # remove chr from each row
            command = ["sed", "'s/^chr//'", path, ">", newVCFPath]
            subprocess.check_call(" ".join(command), shell=True)
            # Empty spaces must be replaced with a period
            command = ["sed", '-i', "'s/\t\t/\t.\t/g'", newVCFPath]
            subprocess.check_call(" ".join(command), shell=True)
            # All INFO/HGVS values have a whitespace,
            # which is not allowed in VCF specs. Replace that with a comma
            command = ['sed', '-i', "'s/ p\./,p./'", newVCFPath]
            subprocess.check_call(" ".join(command), shell=True)
            # Remove empty white space lines (VCF2MAF fails)
            command = ['sed', '-i', "'/^\s*$/d'", newVCFPath]
            subprocess.check_call(" ".join(command), shell=True)
            # Strips out windows indentations \r
            command = ['dos2unix', newVCFPath]
            subprocess.check_call(command)
            vcfCols = ["#CHROM", "POS", "ID", "REF", "ALT",
                       "QUAL", "FILTER", "INFO", "FORMAT"]
            with open(newVCFPath, "r") as f:
                for line in f:
                    if line.startswith("#CHROM"):
                        cols = line

            cols = cols.replace("\n", "")
            cols = cols.replace("\r", "")
            cols = cols.split("\t")

            samples = [i for i in cols if i not in vcfCols]

            # tumorName = vcfName.replace(".vcf","")

            if len(samples) == 1:
                tumor = samples[0]
                normal = "NORMAL"
            elif len(samples) == 2:
                # Assumes that Tumor is always first, normal is second
                tumor = samples[0]
                normal = samples[1]
            else:
                tumor = "TUMOR"
                normal = "NORMAL"
            # If the tumor name isn't TUMOR,
            # set the sample id to be the tumor name
            if tumor == "TUMOR":
                tumorName = vcfName.replace(".vcf", "")
            else:
                tumorName = tumor
            newMAFPath = newVCFPath + ".maf"
            if os.path.isfile(newMAFPath):
                mafFiles.append(newMAFPath)
            else:
                command = ['perl', os.path.join(vcf2mafPath, 'vcf2maf.pl'),
                           '--input-vcf', newVCFPath,
                           '--output-maf', newMAFPath,
                           '--vep-path', veppath,
                           '--vep-data', vepdata,
                           '--vep-forks', '8',
                           '--tumor-id', tumorName,
                           '--normal-id', normal,
                           '--vcf-tumor-id', tumor,
                           '--custom-enst', os.path.join(vcf2mafPath, 'data/isoform_overrides_uniprot')]
                if reference is not None:
                    command.extend(["--ref-fasta", reference])
                subprocess.check_call(command)
                if os.path.isfile(newMAFPath):
                    mafFiles.append(newMAFPath)

        logger.info("MERGING MAFS")
        # maf = pd.DataFrame()
        newMafPath = os.path.join(
            centerStagingFolder,
            "data_mutations_extended_{}_VCF.txt".format(self.center))
        narrowMafPath = os.path.join(
            centerStagingFolder,
            "data_mutations_extended_{}_VCF_narrow.txt".format(self.center))
        open(newMafPath, "w").close()
        open(narrowMafPath, "w").close()
        narrowMafColumns = [
            col['name'] for col in self.syn.getTableColumns(mafSynId)
            if col['name'] != 'inBED']

        narrowMafPaths = [narrowMafPath]
        for mafFile in mafFiles:
            mafDf = pd.read_csv(mafFile, sep="\t", comment="#")
            mafDf.drop_duplicates(inplace=True)
            mafDf = self.formatMAF(mafDf)
            self.createFinalMaf(mafDf, newMafPath)
            narrowMafDf = mafDf[narrowMafColumns]
            maf_statinfo = os.stat(narrowMafPath)
            # This is done because of the table update size limit.
            # Can only update 1 GB at a time
            if maf_statinfo.st_size / 1000000 > 900:
                narrowMafPath = narrowMafPath.replace(
                    ".txt", str(len(narrowMafPaths)) + ".txt")
                narrowMafPaths.append(narrowMafPath)
            self.createFinalMaf(narrowMafDf, narrowMafPath)

        if len(mafFiles) > 0:
            # These functions have to be next to each other,
            # because no modifications can happen
            # Store MAF flat file into synapse
            # First because of 1 GB limit
            self.storeProcessedMaf(newMafPath, mafSynId, centerMafSynId)
            # Store Narrow MAF into db
            # Need to figure out how to fix this 1 GB limit
            for narrow_path in narrowMafPaths:
                self.storeProcessedMaf(
                    narrow_path, mafSynId, centerMafSynId, isNarrow=True)

        return newMafPath

    def process_steps(self, filePath, processing, databaseToSynIdMappingDf,
                      vcf2mafPath, veppath, vepdata, validVCF, path_to_GENIE,
                      reference):
        mutationFiles = []
        if processing == self._fileType:
            mafProcessing = "mafSP" if self._fileType == "mafSP" else 'vcf2maf'
            mafSynId = databaseToSynIdMappingDf.Id[
                databaseToSynIdMappingDf['Database'] == mafProcessing][0]
            centerMafSynId = databaseToSynIdMappingDf.Id[
                databaseToSynIdMappingDf['Database'] == "centerMaf"][0]
            logger.info(validVCF)
            vcfFilePath = self.process_helper(
                validVCF, path_to_GENIE, mafSynId, centerMafSynId,
                vcf2mafPath, veppath, vepdata, reference=reference)
            mutationFiles = [vcfFilePath]
            logger.info("UPDATED DATABASE WITH: %s" % ", ".join(mutationFiles))
        else:
            logger.info(
                "Please run with `--process {filetype}` parameter "
                "if you want to reannotate the {filetype} files".format(
                    filetype=self._fileType))
        return mutationFiles

    def _validate(self, vcfdf):
        '''
        Validates the content of a vcf file

        Args:
            vcfdf: pandas dataframe containing vcf content

        Returns:
            total_error - error messages
            warning - warning messages
        '''
        required_headers = pd.Series(["#CHROM", "POS", "ID", "REF", "ALT",
                                      "QUAL", "FILTER", "INFO"])
        total_error = ""
        warning = ""
        if not all(required_headers.isin(vcfdf.columns)):
            total_error += ("Your vcf file must have these headers: "
                            "CHROM, POS, ID, REF, ALT, QUAL, FILTER, INFO.\n")
        else:
            # No duplicated values
            primary_cols = ["#CHROM", "POS", "ID", "REF", "ALT"]
            if vcfdf.duplicated(primary_cols).any():
                total_error += "Your vcf file should not have duplicate rows\n"

            if vcfdf[['#CHROM', 'POS']].isnull().values.any():
                total_error += ("Your vcf file may contain rows that are "
                                "space delimited instead of tab delimited.\n")

        if len(vcfdf.columns) > 8:
            if "FORMAT" not in vcfdf.columns:
                total_error += ("Your vcf file must have FORMAT header "
                                "if genotype columns exist.\n")

        # Require that they report variants mapped to
        # either GRCh37 or hg19 without
        # the chr-prefix. variants on chrM are not supported
        have_column = process_functions.checkColExist(vcfdf, "#CHROM")
        if have_column:
            nochr = ["chr" in i for i in vcfdf['#CHROM'] if isinstance(i, str)]
            if sum(nochr) > 0:
                warning += ("Your vcf file should not have the chr prefix "
                            "in front of chromosomes.\n")
            if sum(vcfdf['#CHROM'].isin(["chrM"])) > 0:
                total_error += "Your vcf file must not have variants on chrM.\n"

        # No white spaces
        white_space = vcfdf.apply(lambda x: contains_whitespace(x), axis=1)
        if sum(white_space) > 0:
            warning += ("Your vcf file should not have any "
                        "white spaces in any of the columns.\n")

        # I can also recommend a `bcftools query` command that
        # will parse a VCF in a detailed way,
        # and output with warnings or errors if the format is not adhered too
        return total_error, warning



import logging
import os

import pandas as pd

from .example_filetype_format import FileTypeFormat
from . import process_functions

logger = logging.getLogger(__name__)

# def updateMutationInCisData(syn, databaseSynId, newData, center, col, toDelete=False):
#     databaseEnt = syn.get(databaseSynId)
#     database = syn.tableQuery("SELECT * FROM %s where Center ='%s'" % (databaseSynId, center))
#     database = database.asDataFrame()[col]
#     process_functions.updateDatabase(syn, database, newData, databaseSynId, databaseEnt.primaryKey, toDelete)
        

class mutationsInCis(FileTypeFormat):

    _fileType = "mutationsInCis"

    def _get_dataframe(self, filePathList):
        '''
        Mutation In Cis is a csv file
        '''
        filePath = filePathList[0]
        df = pd.read_csv(filePath,comment="#")
        return(df)

    # VALIDATE FILENAME
    def _validateFilename(self, filePath):
        assert os.path.basename(filePath[0]) == "mutationsInCis_filtered_samples.csv"
    
    # PROCESS
    def process_steps(self, mutationInCis, newPath, databaseSynId):
        process_functions.updateData(self.syn, databaseSynId, mutationInCis, self.center, filterByColumn="Center")
        mutationInCis.to_csv(newPath, sep="\t",index=False)
        return(newPath)

    def _validate(self, mutationInCisDf):
        mutationInCisSynId = process_functions.getDatabaseSynId(self.syn, "mutationsInCis", test=self.testing)
        #Pull down the correct database
        existingMergeCheck = self.syn.tableQuery("select * from %s where Center = '%s'" % (mutationInCisSynId,self.center))
        existingMergeCheckDf = existingMergeCheck.asDataFrame()

        total_error = ""
        warning = ""
        REQUIRED_HEADERS = pd.Series(['Flag','Center','Tumor_Sample_Barcode','Hugo_Symbol','HGVSp_Short','Variant_Classification','Chromosome','Start_Position','Reference_Allele','Tumor_Seq_Allele2','t_alt_count_num','t_depth'])
        primaryKeys = ['Tumor_Sample_Barcode', 'HGVSp_Short', 'Start_Position', 'Reference_Allele', 'Tumor_Seq_Allele2']
        if not all(REQUIRED_HEADERS.isin(mutationInCisDf.columns)):
            total_error += "Mutations In Cis Filter File: Must at least have these headers: %s.\n" % ",".join(REQUIRED_HEADERS[~REQUIRED_HEADERS.isin(mutationInCisDf.columns)])
        else:
            new = mutationInCisDf[primaryKeys].fillna("")
            existing = existingMergeCheckDf[primaryKeys].fillna("")

            existing['primaryAll'] = [" ".join(values.astype(str)) for i, values in existing.iterrows()]
            new['primaryAll'] = [" ".join(values.astype(str)) for i, values in new.iterrows()]
            if not all(new.primaryAll.isin(existing.primaryAll)):
                total_error += "Mutations In Cis Filter File: All variants must come from the original mutationInCis_filtered_samples.csv file in each institution's staging folder.\n"
        return(total_error, warning)



import datetime
import logging
import os

import pandas as pd

from .example_filetype_format import FileTypeFormat
from . import process_functions

logger = logging.getLogger(__name__)


class vitalStatus(FileTypeFormat):

    _fileType = "vitalStatus"

    def _validateFilename(self, filePath):
        '''
        Validate filename: vital_status.txt
        '''
        assert os.path.basename(filePath[0]) == "vital_status.txt"

    def _validate(self, vitalStatusDf):
        total_error = ""
        warning = ""

        # PATIENT ID
        haveColumn = \
            process_functions.checkColExist(vitalStatusDf, "PATIENT_ID")
        if haveColumn:
            if vitalStatusDf.PATIENT_ID.isnull().any():
                total_error += (
                    "Vital status file: Please double check your PATIENT_ID"
                    " column. No null values allowed.\n")
        else:
            total_error += "Vital status file: Must have PATIENT_ID column.\n"

        # YEAR DEATH
        haveColumn = \
            process_functions.checkColExist(vitalStatusDf, "YEAR_DEATH")
        if haveColumn:
            notNullYears = \
                vitalStatusDf.YEAR_DEATH[~vitalStatusDf.YEAR_DEATH.isnull()]
            try:
                notNullYears.apply(
                    lambda x: datetime.datetime.strptime(str(int(x)), '%Y'))
            except Exception:
                total_error += (
                    "Vital status file: Please double check your YEAR_DEATH"
                    " column, it must be an integer in YYYY format or an "
                    "empty string.\n")
        else:
            total_error += "Vital status file: Must have YEAR_DEATH column.\n"

        # YEAR CONTACT
        haveColumn = \
            process_functions.checkColExist(vitalStatusDf, "YEAR_CONTACT")
        if haveColumn:
            notNullYears = vitalStatusDf.YEAR_CONTACT[
                ~vitalStatusDf.YEAR_CONTACT.isnull()]
            try:
                notNullYears.apply(
                    lambda x: datetime.datetime.strptime(str(int(x)), '%Y'))
            except Exception:
                total_error += (
                    "Vital status file: Please double check your "
                    "YEAR_CONTACT column, it must be an integer in YYYY "
                    "format or an empty string.\n")
        else:
            total_error += \
                "Vital status file: Must have YEAR_CONTACT column.\n"

        # INT CONTACT
        haveColumn = \
            process_functions.checkColExist(vitalStatusDf, "INT_CONTACT")
        if haveColumn:
            check_ints = [
                process_functions.checkInt(i)
                for i in vitalStatusDf.INT_CONTACT
                if not pd.isnull(i) and i not in ['>32485', '<6570']]
            if not all(check_ints):
                total_error += (
                    "Vital status file: Please double check your "
                    "INT_CONTACT column, it must be an integer, an empty "
                    "string, >32485, or <6570.\n")
        else:
            total_error += "Vital status file: Must have INT_CONTACT column.\n"

        # INT DOD
        haveColumn = process_functions.checkColExist(vitalStatusDf, "INT_DOD")
        if haveColumn:
            check_dod_ints = [
                process_functions.checkInt(i)
                for i in vitalStatusDf.INT_DOD
                if not pd.isnull(i) and i not in ['>32485', '<6570']]
            if not all(check_dod_ints):
                total_error += (
                    "Vital status file: Please double check your "
                    "INT_DOD column, it must be an integer, an empty "
                    "string, >32485, or <6570.\n")
        else:
            total_error += "Vital status file: Must have INT_DOD column.\n"

        haveColumn = process_functions.checkColExist(vitalStatusDf, "DEAD")
        if haveColumn:
            check_boolean = [
                isinstance(i, bool)
                for i in vitalStatusDf.DEAD if not pd.isnull(i)]
            if not all(check_boolean):
                total_error += (
                    "Vital status file: Please double check your DEAD "
                    "column, it must be a boolean value or an empty string.\n")
        else:
            total_error += "Vital status file: Must have DEAD column.\n"

        return(total_error, warning)

    def _process(self, vitalStatusDf):
        vitalStatusDf.PATIENT_ID = [
            process_functions.checkGenieId(patient, self.center)
            for patient in vitalStatusDf.PATIENT_ID]
        vitalStatusDf['CENTER'] = self.center
        return(vitalStatusDf)

    # PROCESS
    def process_steps(self, vitalStatusDf, databaseSynId, newPath):
        vitalStatusDf = self._process(vitalStatusDf)
        process_functions.updateData(
            self.syn, databaseSynId, vitalStatusDf, self.center)
        vitalStatusDf.to_csv(newPath, sep="\t", index=False)
        return(newPath)



#! /usr/bin/env python3
import os
import argparse
import logging

from genie import input_to_database
from genie import write_invalid_reasons
from genie import process_functions

logging.basicConfig(level=logging.INFO)
logger = logging.getLogger(__name__)


def main(process,
         center=None,
         pemfile=None,
         delete_old=False,
         only_validate=False,
         oncotree_link=None,
         create_new_maf_database=False,
         testing=False,
         debug=False,
         reference=None,
         vcf2maf_path=None,
         vep_path=None,
         vep_data=None):

    syn = process_functions.synLogin(pemfile, debug=debug)
    # Must specify correct paths to vcf2maf, VEP and VEP data
    # if trying to process vcf, maf and mafSP
    if process in ['vcf', 'maf', 'mafSP'] and not only_validate:
        assert os.path.exists(vcf2maf_path), (
            "Path to vcf2maf (--vcf2mafPath) must be specified "
            "if `--process {vcf,maf,mafSP}` is used")
        assert os.path.exists(vep_path), (
            "Path to VEP (--vepPath) must be specified "
            "if `--process {vcf,maf,mafSP}` is used")
        assert os.path.exists(vep_data), (
            "Path to VEP data (--vepData) must be specified "
            "if `--process {vcf,maf,mafSP}` is used")

    if testing:
        database_to_synid_mapping_synid = "syn11600968"
    else:
        database_to_synid_mapping_synid = "syn10967259"

    databaseToSynIdMapping = syn.tableQuery(
        'SELECT * FROM {}'.format(database_to_synid_mapping_synid))
    databaseToSynIdMappingDf = databaseToSynIdMapping.asDataFrame()

    center_mapping_id = process_functions.getDatabaseSynId(
        syn, "centerMapping",
        databaseToSynIdMappingDf=databaseToSynIdMappingDf)

    center_mapping = syn.tableQuery('SELECT * FROM %s' % center_mapping_id)
    center_mapping_df = center_mapping.asDataFrame()

    if center is not None:
        assert center in center_mapping_df.center.tolist(), (
            "Must specify one of these centers: {}".format(
                ", ".join(center_mapping_df.center)))
        centers = [center]
    else:
        center_mapping_df = \
            center_mapping_df[~center_mapping_df['inputSynId'].isnull()]
        # release is a bool column
        center_mapping_df = center_mapping_df[center_mapping_df['release']]
        centers = center_mapping_df.center

    if oncotree_link is None:
        onco_link = databaseToSynIdMappingDf['Id'][
            databaseToSynIdMappingDf['Database'] == 'oncotreeLink'].values[0]
        onco_link_ent = syn.get(onco_link)
        oncotree_link = onco_link_ent.externalURL
    # Check if you can connect to oncotree link,
    # if not then don't run validation / processing
    process_functions.checkUrl(oncotree_link)

    center_mapping_ent = syn.get(center_mapping_id)
    if center_mapping_ent.get('isProcessing', ['True'])[0] == 'True':
        raise Exception(
            "Processing/validation is currently happening.  "
            "Please change/add the 'isProcessing' annotation on {} "
            "to False to enable processing".format(center_mapping_id))
    else:
        center_mapping_ent.isProcessing = "True"
        center_mapping_ent = syn.store(center_mapping_ent)
    # remove this query timeout and see what happens
    # syn.table_query_timeout = 50000

    # Create new maf database, should only happen once if its specified
    if create_new_maf_database:
        databaseToSynIdMappingDf = \
            input_to_database.create_and_archive_maf_database(syn, databaseToSynIdMappingDf)

    for center in centers:
        input_to_database.center_input_to_database(
            syn, center, process,
            testing, only_validate,
            vcf2maf_path, vep_path,
            vep_data, databaseToSynIdMappingDf,
            center_mapping_df, reference=reference,
            delete_old=delete_old,
            oncotree_link=oncotree_link)

    # To ensure that this is the new entity
    center_mapping_ent = syn.get(center_mapping_id)
    center_mapping_ent.isProcessing = "False"
    center_mapping_ent = syn.store(center_mapping_ent)

    error_tracker_synid = process_functions.getDatabaseSynId(
        syn, "errorTracker", databaseToSynIdMappingDf=databaseToSynIdMappingDf)
    # Only write out invalid reasons if the center
    # isnt specified and if only validate
    if center is None and only_validate:
        logger.info("WRITING INVALID REASONS TO CENTER STAGING DIRS")
        write_invalid_reasons.write_invalid_reasons(
            syn, center_mapping_df, error_tracker_synid)


if __name__ == "__main__":
    '''
    Argument parsers
    TODO: Fix case of arguments
    '''
    parser = argparse.ArgumentParser(
        description='GENIE center ')
    parser.add_argument(
        "process",
        choices=['vcf', 'maf', 'main', 'mafSP'],
        help='Process vcf, maf or the rest of the files')
    parser.add_argument(
        '--center',
        help='The centers')
    parser.add_argument(
        "--pemFile",
        type=str,
        help="Path to PEM file (genie.pem)")
    parser.add_argument(
        "--deleteOld",
        action='store_true',
        help="Delete all old processed and temp files")
    parser.add_argument(
        "--onlyValidate",
        action='store_true',
        help="Only validate the files, don't process")
    parser.add_argument(
        "--oncotree_link",
        type=str,
        help="Link to oncotree code")
    parser.add_argument(
        "--createNewMafDatabase",
        action='store_true',
        help="Creates a new maf database")
    parser.add_argument(
        "--testing",
        action='store_true',
        help="Testing the infrastructure!")
    parser.add_argument(
        "--debug",
        action='store_true',
        help="Add debug mode to synapse")
    parser.add_argument(
        "--reference",
        type=str,
        help="Path to VCF reference file")

    # DEFAULT PARAMS
    parser.add_argument(
        "--vcf2mafPath",
        type=str,
        help="Path to vcf2maf",
        default=os.path.expanduser("~/vcf2maf-1.6.14"))
    parser.add_argument(
        "--vepPath",
        type=str,
        help="Path to VEP",
        default=os.path.expanduser("~/vep"))
    parser.add_argument(
        "--vepData",
        type=str,
        help="Path to VEP data",
        default=os.path.expanduser("~/.vep"))

    args = parser.parse_args()

    main(args.process,
         center=args.center,
         pemfile=args.pemFile,
         delete_old=args.deleteOld,
         only_validate=args.onlyValidate,
         oncotree_link=args.oncotree_link,
         create_new_maf_database=args.createNewMafDatabase,
         testing=args.testing,
         debug=args.debug,
         reference=args.reference,
         vcf2maf_path=args.vcf2mafPath,
         vep_path=args.vepPath,
         vep_data=args.vepData)



#!/usr/bin/env python
"""
Creates case lists per cancer type
"""
import argparse
import csv
import os

CASE_LIST_TEXT_TEMPLATE = (
    'cancer_study_identifier: {study_id}\n'
    'stable_id: {stable_id}\n'
    'case_list_name: {case_list_name}\n'
    'case_list_description: {case_list_description}\n'
    'case_list_ids: {case_list_ids}')


def create_case_lists_map(clinical_file_name):
    """
    Creates the case list dictionary

    Args:
        clinical_file_name: clinical file path

    Returns:
        dict: key = cancer_type
              value = list of sample ids
    """
    with open(clinical_file_name, 'rU') as clinical_file:
        clinical_file_map = {}
        reader = csv.DictReader(clinical_file, dialect='excel-tab')
        samples = []
        for row in reader:
            if row['CANCER_TYPE'] not in clinical_file_map:
                clinical_file_map[row['CANCER_TYPE']] = [row['SAMPLE_ID']]
            else:
                clinical_file_map[row['CANCER_TYPE']].append(row['SAMPLE_ID'])
            samples.append(row['SAMPLE_ID'])
    return clinical_file_map, samples


def _write_single_oncotree_case_list(cancer_type, ids, study_id,
                                     output_directory):
    """
    Writes one oncotree case list. Python verisons below
    3.6 will sort the dictionary keys which causes tests to fail

    Args:
        cancer_type: Oncotree code cancer type
        ids: GENIE sample ids
        study_id: cBioPortal study id
        output_directory: case list output directory

    Returns:
        case list file path
    """
    cancer_type = 'NA' if cancer_type == '' else cancer_type
    cancer_type_no_spaces = \
        cancer_type.replace(' ', '_').replace(',', '').replace("/", "_")
    cancer_type_no_spaces = 'no_oncotree_code' \
        if cancer_type_no_spaces == 'NA' else cancer_type_no_spaces
    case_list_text = CASE_LIST_TEXT_TEMPLATE.format(
        study_id=study_id,
        stable_id=study_id + '_' + cancer_type_no_spaces,
        case_list_name='Tumor Type: ' + cancer_type,
        case_list_description='All tumors with cancer type ' + cancer_type,
        case_list_ids='\t'.join(ids))
    case_list_path = os.path.abspath(os.path.join(
        output_directory, 'cases_' + cancer_type_no_spaces + '.txt'))
    with open(case_list_path, 'w') as case_list_file:
        case_list_file.write(case_list_text)
    return case_list_path


def write_case_list_files(clinical_file_map, output_directory, study_id):
    """
    Writes the cancer_type case list file to case_lists directory

    Args:
        clinical_file_map: cancer type to sample id mapping from
                           create_case_lists_map
        output_directory: Directory to write case lists
        study_id: cBioPortal study id

    Returns:
        list: oncotree code case list files
    """
    case_list_files = []
    for cancer_type, ids in clinical_file_map.items():
        case_list_path = _write_single_oncotree_case_list(
            cancer_type, ids, study_id, output_directory)
        case_list_files.append(case_list_path)
    return case_list_files


def create_sequenced_samples(gene_matrix_file_name):
    """
    Gets samples sequenced

    Args:
        gene_matrix_file_name: Gene matrix file

    Returns:
        tuple: list of clinical samples and cna samples
    """
    with open(gene_matrix_file_name, 'r') as gene_matrix_file:
        reader = csv.DictReader(gene_matrix_file, dialect='excel-tab')
        clinical_samples = []
        cna_samples = []
        for row in reader:
            if row['cna'] != "NA":
                cna_samples.append(row['SAMPLE_ID'])
            clinical_samples.append(row['SAMPLE_ID'])

    return clinical_samples, cna_samples


def write_case_list_sequenced(clinical_samples, output_directory, study_id):
    """
    Writes the genie sequenced and all samples. Since all samples
    are sequenced, _all and _sequenced are equal

    Args:
        clinical_samples: List of clinical samples
        output_directory: Directory to write case lists
        study_id: cBioPortal study id

    Returns:
        list: case list sequenced and all
    """
    caselist_files = []
    case_list_ids = '\t'.join(clinical_samples)
    case_sequenced_path = os.path.abspath(os.path.join(output_directory,
                                                       'cases_sequenced.txt'))
    with open(case_sequenced_path, 'w') as case_list_sequenced_file:
        case_list_file_text = CASE_LIST_TEXT_TEMPLATE.format(
            study_id=study_id,
            stable_id=study_id + '_sequenced',
            case_list_name='Sequenced Tumors',
            case_list_description='All sequenced samples',
            case_list_ids=case_list_ids)
        case_list_sequenced_file.write(case_list_file_text)
    cases_all_path = os.path.abspath(os.path.join(output_directory,
                                                  'cases_all.txt'))
    with open(cases_all_path, 'w') as case_list_all_file:
        case_list_file_text = CASE_LIST_TEXT_TEMPLATE.format(
            study_id=study_id,
            stable_id=study_id + '_all',
            case_list_name='All samples',
            case_list_description='All samples',
            case_list_ids=case_list_ids)
        case_list_all_file.write(case_list_file_text)
    caselist_files.extend([case_sequenced_path, cases_all_path])
    return caselist_files


def write_case_list_cna(cna_samples, output_directory, study_id):
    """
    Writes the cna sequenced samples

    Args:
        cna_samples: List of cna samples
        output_directory: Directory to write case lists
        study_id: cBioPortal study id

    Returns:
        cna caselist path
    """
    case_list_ids = '\t'.join(cna_samples)
    cna_caselist_path = os.path.abspath(os.path.join(
        output_directory, 'cases_cna.txt'))
    with open(cna_caselist_path, 'w') as case_list_file:
        case_list_file_text = CASE_LIST_TEXT_TEMPLATE.format(
            study_id=study_id,
            stable_id=study_id + '_cna',
            case_list_name='Samples with CNA',
            case_list_description='Samples with CNA',
            case_list_ids=case_list_ids)
        case_list_file.write(case_list_file_text)
    return cna_caselist_path


def write_case_list_cnaseq(cna_samples, output_directory, study_id):
    """
    writes both cna and mutation samples (Just _cna file for now)

    Args:
        cna_samples: List of cna samples
        output_directory: Directory to write case lists
        study_id: cBioPortal study id

    Returns:
        cnaseq path
    """
    case_list_ids = '\t'.join(cna_samples)
    cnaseq_caselist_path = os.path.abspath(os.path.join(
        output_directory, 'cases_cnaseq.txt'))
    with open(cnaseq_caselist_path, 'w') as case_list_file:
        case_list_file_text = CASE_LIST_TEXT_TEMPLATE.format(
            study_id=study_id,
            stable_id=study_id + '_cnaseq',
            case_list_name='Samples with CNA and mutation',
            case_list_description='Samples with CNA and mutation',
            case_list_ids=case_list_ids)
        case_list_file.write(case_list_file_text)
    return cnaseq_caselist_path


def main(clinical_file_name,
         gene_matrix_file_name,
         output_directory,
         study_id):
    '''
    Gets clinical file and gene matrix file and processes it
    to obtain case list files

    Args:
        clinical_file_name: Clinical file path
        gene_matrix_file_name: Gene matrix file path
        output_directory: Output directory of case list files
        study_id: cBioPortal study id
    '''
    case_lists_map, clin_samples = create_case_lists_map(clinical_file_name)
    write_case_list_files(case_lists_map, output_directory, study_id)
    # create_sequenced_samples used to get the samples, but since the removal
    # of WES samples, we can no longer rely on the gene matrix file to grab
    # all sequenced samples
    # TODO: cna_samples will be an issue once we have CNA data from WES panels
    _, cna_samples = create_sequenced_samples(gene_matrix_file_name)
    write_case_list_sequenced(clin_samples, output_directory, study_id)
    write_case_list_cna(cna_samples, output_directory, study_id)
    write_case_list_cnaseq(cna_samples, output_directory, study_id)


if __name__ == '__main__':
    parser = argparse.ArgumentParser(description='Creation of case lists') #pylint: disable=invalid-name

    parser.add_argument("clinical_file_name",
                        type=str,
                        help="Clinical file path")
    parser.add_argument("gene_matrix_file_name",
                        type=str,
                        help="gene matrix file path")
    parser.add_argument("output_dir",
                        type=str,
                        help="Output directory")
    parser.add_argument("study_id",
                        type=str,
                        help="Output directory")
    args = parser.parse_args() #pylint: disable=invalid-name

    main(args.clinical_file_name,
         args.gene_matrix_file_name,
         args.output_dir,
         args.study_id)



import logging
import os

from .example_filetype_format import FileTypeFormat
from . import process_functions

logger = logging.getLogger(__name__)


class clinicalSP(FileTypeFormat):

    _fileType = "clinicalSP"

    # VALIDATE FILENAME
    def _validateFilename(self, filePath):
        assert os.path.basename(filePath[0]) == "nonGENIE_data_clinical.txt"

    def _process(self, clinicalSPDf):
        clinicalSPDf['SAMPLE_ID'] = [
            process_functions.checkGenieId(sample, self.center)
            for sample in clinicalSPDf['SAMPLE_ID']]
        clinicalSPDf['CENTER'] = self.center
        clinicalSPDf['PATIENT_ID'] = [
            process_functions.checkGenieId(sample, self.center)
            for sample in clinicalSPDf['PATIENT_ID']]
        return(clinicalSPDf)

    def process_steps(self, clinicalSPDf, newPath, databaseSynId):
        clinicalSPDf = self._process(clinicalSPDf)
        process_functions.updateData(
            self.syn, databaseSynId, clinicalSPDf, self.center)
        clinicalSPDf.to_csv(newPath, sep="\t", index=False)
        return(newPath)

    def _validate(self, clinicalDF):
        clinicalDF.columns = [col.upper() for col in clinicalDF.columns]
        # clinicalDF = clinicalDF.fillna("")
        total_error = ""
        warning = ""

        # CHECK: SAMPLE_ID
        haveColumn = process_functions.checkColExist(clinicalDF, 'SAMPLE_ID')
        if not haveColumn:
            total_error += (
                "nonGENIE_data_clinical.txt: "
                "File must have SAMPLE_ID column.\n")
        else:
            if sum(clinicalDF['SAMPLE_ID'].isnull()) > 0:
                total_error += (
                    "nonGENIE_data_clinical.txt: "
                    "There can't be any blank values for SAMPLE_ID\n")

        # CHECK: SEQ_ASSAY_ID
        haveColumn = process_functions.checkColExist(
            clinicalDF, "SEQ_ASSAY_ID")
        if haveColumn:
            if sum(clinicalDF['SEQ_ASSAY_ID'].isnull()) > 0:
                warning += (
                    "nonGENIE_data_clinical.txt: Please double check "
                    "your SEQ_ASSAY_ID columns, there are empty rows.\n")
        else:
            total_error += (
                "nonGENIE_data_clinical.txt: "
                "File must have SEQ_ASSAY_ID column.\n")

        # CHECK: PATIENT_ID
        haveColumn = process_functions.checkColExist(clinicalDF, 'PATIENT_ID')
        if not haveColumn:
            total_error += (
                "nonGENIE_data_clinical.txt: "
                "File must have PATIENT_ID column.\n")
        else:
            if sum(clinicalDF['PATIENT_ID'].isnull()) > 0:
                total_error += (
                    "nonGENIE_data_clinical.txt: "
                    "There can't be any blank values for PATIENT_ID\n")
        return(total_error, warning)



#!/usr/bin/env python3
import importlib
import inspect
import logging
import sys

import synapseclient
try:
    from synapseclient.core.exceptions import SynapseHTTPError
except ModuleNotFoundError:
    from synapseclient.exceptions import SynapseHTTPError

from . import config
from . import example_filetype_format
from . import process_functions

logging.basicConfig()
logger = logging.getLogger(__name__)
logger.setLevel(logging.INFO)


class ValidationHelper(object):

    # Used for the kwargs in validate_single_file
    # Overload this per class
    _validate_kwargs = []

    def __init__(self, syn, center, entitylist,
                 format_registry=config.PROCESS_FILES,
                 testing=False, file_type=None):

        """A validator helper class for a center's files.

        Args:
            syn: a synapseclient.Synapse object
            center: The participating center name.
            filepathlist: a list of file paths.
            format_registry: A dictionary mapping file format name to the
                             format class.
            testing: Run in testing mode.
            file_type: Specify file type to skip filename validation
        """

        self._synapse_client = syn
        self.entitylist = entitylist
        self.center = center
        self._format_registry = format_registry
        self.testing = testing
        self.file_type = (self.determine_filetype()
                          if file_type is None else file_type)

    def determine_filetype(self):
        """Gets the file type of the file by validating its filename

        Args:
            syn: Synapse object
            filepathlist: list of filepaths to center files

        Returns:
            str: File type of input files.  None if no filetype found

        """
        filetype = None
        # Loop through file formats
        for file_format in self._format_registry:
            validator = self._format_registry[file_format](self._synapse_client, self.center,
                                                           testing=self.testing)
            try:
                filenames = [entity.name for entity in self.entitylist]
                filetype = validator.validateFilename(filenames)
            except AssertionError:
                continue
            # If valid filename, return file type.
            if filetype is not None:
                break
        return filetype

    def validate_single_file(self, **kwargs):
        """Validate a submitted file unit.

        Returns:
            message: errors and warnings
            valid: Boolean value of validation status
        """

        if self.file_type not in self._format_registry:
            valid = False
            errors = "Your filename is incorrect! Please change your filename before you run the validator or specify --filetype if you are running the validator locally"
            warnings = ""
        else:
            mykwargs = {}
            for required_parameter in self._validate_kwargs:
                assert required_parameter in kwargs.keys(), \
                    "%s not in parameter list" % required_parameter
                mykwargs[required_parameter] = kwargs[required_parameter]

            validator_cls = self._format_registry[self.file_type]
            validator = validator_cls(self._synapse_client, self.center,
                                      testing=self.testing)
            filepathlist = [entity.path for entity in self.entitylist]
            valid, errors, warnings = validator.validate(filePathList=filepathlist,
                                                         **mykwargs)

        # Complete error message
        message = collect_errors_and_warnings(errors, warnings)

        return (valid, message)


class GenieValidationHelper(ValidationHelper):
    """A validator helper class for AACR Project Genie.
    """

    _validate_kwargs = ['oncotree_link', 'nosymbol_check']


def collect_errors_and_warnings(errors, warnings):
    '''Aggregates error and warnings into a string.

    Args:
        errors: string of file errors, separated by new lines.
        warnings: string of file warnings, separated by new lines.

    Returns:
        message - errors + warnings
    '''
    # Complete error message
    message = "----------------ERRORS----------------\n"
    if errors == "":
        message = "YOUR FILE IS VALIDATED!\n"
        logger.info(message)
    else:
        for error in errors.split("\n"):
            if error != '':
                logger.error(error)
        message += errors
    if warnings != "":
        for warning in warnings.split("\n"):
            if warning != '':
                logger.warning(warning)
        message += "-------------WARNINGS-------------\n" + warnings
    return message


def get_config(syn, synid):
    """Gets Synapse database to Table mapping in dict

    Args:
        syn: Synapse connection
        synid: Synapse id of database mapping table

    Returns:
        dict: {'databasename': 'synid'}

    """
    config = syn.tableQuery('SELECT * FROM {}'.format(synid))
    configdf = config.asDataFrame()
    configdf.index = configdf['Database']
    config_dict = configdf.to_dict()
    return config_dict['Id']


def _check_parentid_permission_container(syn, parentid):
    """Checks permission / container
    # TODO: Currently only checks if a user has READ permissions
    """
    if parentid is not None:
        try:
            syn_ent = syn.get(parentid, downloadFile=False)
            # If not container, throw an assertion
            assert synapseclient.entity.is_container(syn_ent)
        except (SynapseHTTPError, AssertionError):
            raise ValueError(
                "Provided Synapse id must be your input folder Synapse id "
                "or a Synapse Id of a folder inside your input directory")


def _check_center_input(center, center_list):
    """Checks center input

    Args:
        center: Center name
        center_list: List of allowed centers

    Raises:
        ValueError: If specify a center not part of the center list

    """
    if center not in center_list:
        raise ValueError("Must specify one of these "
                         f"centers: {', '.join(center_list)}")


def _get_oncotreelink(syn, databasetosynid_mappingdf, oncotree_link=None):
    """
    Gets oncotree link unless a link is specified by the user

    Args:
        syn: Synapse object
        databasetosynid_mappingdf: database to synid mapping
        oncotree_link: link to oncotree. Default is None

    Returns:
        oncotree link

    """
    if oncotree_link is None:
        oncolink = databasetosynid_mappingdf.query(
            'Database == "oncotreeLink"').Id
        oncolink_ent = syn.get(oncolink.iloc[0])
        oncotree_link = oncolink_ent.externalURL
    return oncotree_link


def _upload_to_synapse(syn, filepaths, valid, parentid=None):
    """
    Upload to synapse if parentid is specified and valid

    Args:
        syn: Synapse object
        filepaths: List of file paths
        valid: Boolean value for validity of file
        parentid: Synapse id of container. Default is None

    """
    if parentid is not None and valid:
        logger.info("Uploading file to {}".format(parentid))
        for path in filepaths:
            file_ent = synapseclient.File(path, parent=parentid)
            ent = syn.store(file_ent)
            logger.info("Stored to {}".format(ent.id))


def collect_format_types(package_names):
    """Finds subclasses of the example_filetype_format.FileTypeFormat
    from a list of package names.

    Args:
        package_names: A list of Python package names as strings.

    Returns:
        A list of classes that are in the named packages and subclasses
        of example_filetype_format.FileTypeFormat

    """

    file_format_list = []
    for package_name in package_names:
        importlib.import_module(package_name)

    for cls in config.get_subclasses(example_filetype_format.FileTypeFormat):
        logger.debug("checking {}.".format(cls))
        cls_module_name = cls.__module__
        cls_pkg = cls_module_name.split('.')[0]
        if cls_pkg in package_names:
            file_format_list.append(cls)
    file_format_dict = config.make_format_registry_dict(file_format_list)
    return file_format_dict


def _perform_validate(syn, args):
    """This is the main entry point to the genie command line tool."""

    # Check parentid argparse
    _check_parentid_permission_container(syn, args.parentid)

    databasetosynid_mappingdf = process_functions.get_synid_database_mappingdf(
        syn, test=args.testing)

    synid = databasetosynid_mappingdf.query('Database == "centerMapping"').Id

    center_mapping = syn.tableQuery('select * from {}'.format(synid.iloc[0]))
    center_mapping_df = center_mapping.asDataFrame()

    # Check center argparse
    _check_center_input(args.center, center_mapping_df.center.tolist())

    args.oncotree_link = _get_oncotreelink(syn, databasetosynid_mappingdf,
                                           oncotree_link=args.oncotree_link)

    format_registry = collect_format_types(args.format_registry_packages)
    logger.debug("Using {} file formats.".format(format_registry))
    entity_list = [synapseclient.File(name=filepath, path=filepath,
                                      parentId=None)
                   for filepath in args.filepath]
    validator = GenieValidationHelper(syn=syn, center=args.center,
                                      entitylist=entity_list,
                                      format_registry=format_registry,
                                      file_type=args.filetype)
    mykwargs = dict(oncotree_link=args.oncotree_link,
                    nosymbol_check=args.nosymbol_check)
    valid, message = validator.validate_single_file(**mykwargs)

    # Upload to synapse if parentid is specified and valid
    _upload_to_synapse(syn, args.filepath, valid, parentid=args.parentid)



import argparse
import datetime
import logging
import os
import re
import shutil
import subprocess
import time

import synapseclient
import synapseutils
import pandas as pd

from . import process_functions
from . import database_to_staging
from . import create_case_lists
from . import dashboard_table_updater

logging.basicConfig()
logger = logging.getLogger(__name__)
logger.setLevel(logging.INFO)

def storeFile(syn, filePath, parentId, anonymizeCenterDf,
              genie_version, name=None):
    #process.center_anon(filePath, anonymizeCenterDf)
    if name is None:
        name = os.path.basename(filePath)
    return(syn.store(synapseclient.File(filePath, name=name, parent = parentId, versionComment=genie_version)))
    #process.center_convert_back(filePath, anonymizeCenterDf)

#This is the only filter that returns mutation columns to keep
def commonVariantFilter(mafDf):
    mafDf['FILTER'] = mafDf['FILTER'].fillna("")
    toKeep = ["common_variant" not in i for i in mafDf['FILTER']]
    mafDf = mafDf[toKeep]
    return(mafDf)

def consortiumToPublic(syn, processingDate, genie_version, releaseId, databaseSynIdMappingDf, publicReleaseCutOff=365, staging=False):

    ANONYMIZE_CENTER = syn.tableQuery('SELECT * FROM syn10170510')
    ANONYMIZE_CENTER_DF = ANONYMIZE_CENTER.asDataFrame()
    CNA_PATH = os.path.join(database_to_staging.GENIE_RELEASE_DIR,"data_CNA_%s.txt" % genie_version)
    CLINICAL_PATH = os.path.join(database_to_staging.GENIE_RELEASE_DIR,'data_clinical_%s.txt' % genie_version)
    CLINICAL_SAMPLE_PATH = os.path.join(database_to_staging.GENIE_RELEASE_DIR,'data_clinical_sample_%s.txt' % genie_version)
    CLINICAL_PATIENT_PATH = os.path.join(database_to_staging.GENIE_RELEASE_DIR,'data_clinical_patient_%s.txt' % genie_version)
    DATA_GENE_PANEL_PATH = os.path.join(dbTodatabase_to_stagingstaging.GENIE_RELEASE_DIR,'data_gene_matrix_%s.txt' % genie_version)
    MUTATIONS_PATH = os.path.join(database_to_staging.GENIE_RELEASE_DIR,'data_mutations_extended_%s.txt' % genie_version)
    FUSIONS_PATH = os.path.join(database_to_staging.GENIE_RELEASE_DIR,'data_fusions_%s.txt' % genie_version)
    SEG_PATH = os.path.join(database_to_staging.GENIE_RELEASE_DIR,'genie_public_data_cna_hg19_%s.seg' % genie_version)
    COMBINED_BED_PATH = os.path.join(database_to_staging.GENIE_RELEASE_DIR,'genomic_information_%s.txt' % genie_version)

    if not os.path.exists(database_to_staging.GENIE_RELEASE_DIR):
        os.mkdir(database_to_staging.GENIE_RELEASE_DIR)
    if not os.path.exists(database_to_staging.CASE_LIST_PATH):
        os.mkdir(database_to_staging.CASE_LIST_PATH)

    # if staging:
    #   #public release staging
    #   PUBLIC_RELEASE_PREVIEW = "syn7871696"
    #   PUBLIC_RELEASE_PREVIEW_CASELIST = "syn9689659"
    # else:
    #public release preview
    PUBLIC_RELEASE_PREVIEW =  databaseSynIdMappingDf['Id'][databaseSynIdMappingDf['Database'] == 'public'].values[0]
    PUBLIC_RELEASE_PREVIEW_CASELIST = database_to_staging.find_caselistid(syn, PUBLIC_RELEASE_PREVIEW)

    ##############################################################################################################################
    ## Sponsored projects filter
    ##############################################################################################################################
    ## if before release date -> go into staging consortium
    ## if after date -> go into public 
    # sponsoredReleaseDate = syn.tableQuery('SELECT * FROM syn8545108')
    # sponsoredReleaseDateDf = sponsoredReleaseDate.asDataFrame()
    # sponsoredProjectSamples = syn.tableQuery('SELECT * FROM syn8545106')
    # sponsoredProjectSamplesDf = sponsoredProjectSamples.asDataFrame()
    # sponsoredProjectsDf = sponsoredProjectSamplesDf.merge(sponsoredReleaseDateDf, left_on="sponsoredProject", right_on="sponsoredProjects")
    # dates = sponsoredProjectsDf['releaseDate'].apply(lambda date: datetime.datetime.strptime(date, '%b-%Y'))
    # publicReleaseSamples = sponsoredProjectsDf['genieSampleId'][dates < processingDate]
    ##############################################################################################################################
    
    # SEQ_DATE filter
    # Jun-2015, given processing date (today) -> public release (processing date - Jun-2015 > 12 months)
    consortiumReleaseWalk = synapseutils.walk(syn, releaseId)

    consortiumRelease = next(consortiumReleaseWalk)
    clinical = [syn.get(synid, followLink=True) for filename, synid in consortiumRelease[2] if filename == "data_clinical.txt"][0]
    gene_matrix = [syn.get(synid, followLink=True) for filename, synid in consortiumRelease[2] if filename == "data_gene_matrix.txt"][0]

    clinicalDf = pd.read_csv(clinical.path, sep="\t", comment="#")
    gene_matrixdf = pd.read_csv(gene_matrix.path, sep="\t")

    removeForPublicSamples = process_functions.seqDateFilter(clinicalDf,processingDate,publicReleaseCutOff)
    #comment back in when public release filter back on
    #publicReleaseSamples = publicReleaseSamples.append(keepForPublicSamples)
    #Make sure all null oncotree codes are removed
    clinicalDf = clinicalDf[~clinicalDf['ONCOTREE_CODE'].isnull()]
    publicReleaseSamples = clinicalDf.SAMPLE_ID[~clinicalDf.SAMPLE_ID.isin(removeForPublicSamples)]

    logger.info("SEQ_DATES for public release: " + ", ".join(set(clinicalDf.SEQ_DATE[clinicalDf.SAMPLE_ID.isin(publicReleaseSamples)].astype(str))))

    #Clinical release scope filter
    #If consortium -> Don't release to public
    clinicalReleaseScope = syn.tableQuery("SELECT * FROM syn8545211 where releaseScope = 'public'")
    publicRelease = clinicalReleaseScope.asDataFrame()

    allClin = clinicalDf[clinicalDf['SAMPLE_ID'].isin(publicReleaseSamples)]
    allClin.to_csv(CLINICAL_PATH, sep="\t", index=False)

    gene_matrixdf = gene_matrixdf[gene_matrixdf['SAMPLE_ID'].isin(publicReleaseSamples)]
    gene_matrixdf.to_csv(DATA_GENE_PANEL_PATH,sep="\t",index=False)
    storeFile(syn, DATA_GENE_PANEL_PATH, PUBLIC_RELEASE_PREVIEW, ANONYMIZE_CENTER_DF, genie_version, name="data_gene_matrix.txt")
    storeFile(syn, CLINICAL_PATH, PUBLIC_RELEASE_PREVIEW, ANONYMIZE_CENTER_DF, genie_version, name="data_clinical.txt")
    
    create_case_lists.main(CLINICAL_PATH, DATA_GENE_PANEL_PATH, database_to_staging.CASE_LIST_PATH, "genie_public")

    caseListFiles = os.listdir(database_to_staging.CASE_LIST_PATH)
    caseListEntities = []
    for casePath in caseListFiles:
        casePath = os.path.join(database_to_staging.CASE_LIST_PATH, casePath)
        caseListEntities.append(storeFile(syn, casePath, PUBLIC_RELEASE_PREVIEW_CASELIST, ANONYMIZE_CENTER_DF, genie_version))

    #Grab mapping table to fill in clinical headers
    mapping_table = syn.tableQuery('SELECT * FROM syn9621600')
    mapping = mapping_table.asDataFrame()
    genePanelEntities = []
    for entName, entId in consortiumRelease[2]:
        if "data_linear" in entName or "meta_" in entName:
            continue
        elif entName == "data_clinical.txt":
            patientCols = publicRelease['fieldName'][publicRelease['level'] == "patient"].tolist()
            sampleCols = ["PATIENT_ID"]
            sampleCols.extend(publicRelease['fieldName'][publicRelease['level'] == "sample"].tolist())
            #clinicalDf is defined on line 36
            # clinicalDf['AGE_AT_SEQ_REPORT'] = [int(math.floor(int(float(i))/365.25)) if process.checkInt(i) else i for i in clinicalDf['AGE_AT_SEQ_REPORT']]
            # clinicalDf['AGE_AT_SEQ_REPORT'][clinicalDf['AGE_AT_SEQ_REPORT'] == ">32485"] = ">89"
            # clinicalDf['AGE_AT_SEQ_REPORT'][clinicalDf['AGE_AT_SEQ_REPORT'] == "<6570"] = "<18"

            clinicalDf = clinicalDf[clinicalDf['SAMPLE_ID'].isin(publicReleaseSamples)]

            #Delete columns that are private scope
            # for private in privateRelease:
            #   del clinicalDf[private]
            process_functions.addClinicalHeaders(clinicalDf, mapping, patientCols, sampleCols, CLINICAL_SAMPLE_PATH, CLINICAL_PATIENT_PATH)

            storeFile(syn, CLINICAL_SAMPLE_PATH, PUBLIC_RELEASE_PREVIEW, ANONYMIZE_CENTER_DF, genie_version, name="data_clinical_sample.txt")
            storeFile(syn, CLINICAL_PATIENT_PATH, PUBLIC_RELEASE_PREVIEW, ANONYMIZE_CENTER_DF, genie_version, name="data_clinical_patient.txt")

        elif "mutation" in entName:
            mutation = syn.get(entId, followLink=True)
            mutationDf = pd.read_csv(mutation.path, sep="\t", comment="#")
            mutationDf = commonVariantFilter(mutationDf)
            mutationDf['FILTER'] = "PASS"
            mutationDf = mutationDf[mutationDf['Tumor_Sample_Barcode'].isin(publicReleaseSamples)]
            text = process_functions.removeFloat(mutationDf)
            with open(MUTATIONS_PATH, 'w') as f:
                f.write(text)
            storeFile(syn, MUTATIONS_PATH, PUBLIC_RELEASE_PREVIEW, ANONYMIZE_CENTER_DF, genie_version, name="data_mutations_extended.txt")

        elif "fusion" in entName:
            fusion = syn.get(entId, followLink=True)
            fusionDf = pd.read_csv(fusion.path, sep="\t")
            #remove = ["Entrez_Gene_Id","Method"]
            #fusionDf = fusionDf[fusionDf.columns[~fusionDf.columns.isin(remove)]]
            fusionDf = fusionDf[fusionDf['Tumor_Sample_Barcode'].isin(publicReleaseSamples)]
            fusionDf.to_csv(FUSIONS_PATH,sep="\t",index=False)
            storeFile(syn, FUSIONS_PATH, PUBLIC_RELEASE_PREVIEW, ANONYMIZE_CENTER_DF, genie_version, name="data_fusions.txt")
        elif "CNA" in entName:
            cna = syn.get(entId, followLink=True)
            cnaDf = pd.read_csv(cna.path, sep="\t")
            cnaDf = cnaDf[cnaDf.columns[cnaDf.columns.isin(publicReleaseSamples.append(pd.Series("Hugo_Symbol")))]]
            text = process_functions.removeFloat(cnaDf)
            text = text.replace("\t\t","\tNA\t").replace("\t\t","\tNA\t").replace('\t\n',"\tNA\n")
            with open(CNA_PATH, "w") as cnaFile:
                cnaFile.write(text)
            storeFile(syn, CNA_PATH, PUBLIC_RELEASE_PREVIEW, ANONYMIZE_CENTER_DF, genie_version, name="data_CNA.txt")
        elif entName.endswith(".seg"):
            seg = syn.get(entId, followLink=True)
            segDf = pd.read_csv(seg.path, sep="\t")
            segDf = segDf[segDf['ID'].isin(publicReleaseSamples)]
            text = process_functions.removeFloat(segDf)
            with open(SEG_PATH, "w") as segFile:
                segFile.write(text)
            storeFile(syn, SEG_PATH, PUBLIC_RELEASE_PREVIEW, ANONYMIZE_CENTER_DF, genie_version, name="genie_public_data_cna_hg19.seg")
        elif entName == "data_gene_matrix.txt":
            pass
            # This file was processed above because it had to be used for generating caselists
            # panel = syn.get(entId, followLink=True)
            # panelDf = pd.read_csv(panel.path, sep="\t")
            # panelDf = panelDf[panelDf['SAMPLE_ID'].isin(publicReleaseSamples)]
            # panelDf.to_csv(DATA_GENE_PANEL_PATH,sep="\t",index=False)
            # storeFile(syn, DATA_GENE_PANEL_PATH, PUBLIC_RELEASE_PREVIEW, ANONYMIZE_CENTER_DF, genie_version, name="data_gene_matrix.txt")
        elif entName == "genomic_information.txt":
            bed = syn.get(entId, followLink=True)
            bedDf = pd.read_csv(bed.path, sep="\t")
            bedDf = bedDf[bedDf.SEQ_ASSAY_ID.isin(allClin.SEQ_ASSAY_ID)]
            bedDf.to_csv(COMBINED_BED_PATH,sep="\t",index=False)
            storeFile(syn, COMBINED_BED_PATH, PUBLIC_RELEASE_PREVIEW, ANONYMIZE_CENTER_DF, genie_version, name="genomic_information.txt")
        elif entName in ["data_clinical_sample.txt", "data_clinical_patient.txt"] or entName.endswith(".html"):
            continue
        elif entName.startswith("data_gene_panel"):
            genePanel = syn.get(entId, followLink=True)
            #Create new gene panel naming and store
            fileName = os.path.basename(genePanel.path)
            newFileList = fileName.split("_")
            newFileList[-1] = genie_version + ".txt"
            newFileName = "_".join(newFileList)
            genePanelPath = os.path.join(database_to_staging.GENIE_RELEASE_DIR, newFileName)
            shutil.copy(genePanel.path, genePanelPath)
            del newFileList[-1]
            entName = "_".join(newFileList)
            entName = entName + ".txt"
            genePanelEntities.append(storeFile(syn, genePanelPath, PUBLIC_RELEASE_PREVIEW, ANONYMIZE_CENTER_DF, genie_version, name=entName))
        else:
            ent = syn.get(entId, followLink=True, downloadFile=False)
            copiedId = synapseutils.copy(syn, ent, PUBLIC_RELEASE_PREVIEW, version=ent.versionNumber, updateExisting=True, setProvenance = None, skipCopyAnnotations=True)
            copiedEnt = syn.get(copiedId[ent.id],downloadFile=False)
            #Set version comment
            copiedEnt.versionComment=genie_version
            syn.store(copiedEnt, forceVersion=False)
    return((caseListEntities,genePanelEntities))

def perform_consortiumToPublic(syn, args, databaseSynIdMappingDf):
    try:
        processingDate = datetime.datetime.strptime(args.processingDate, '%b-%Y')
    except ValueError as e:
        raise ValueError("Process date must be in the format abbreviated_month-YEAR ie. Oct-2017")
    return(consortiumToPublic(syn, processingDate, args.genieVersion, args.releaseId, databaseSynIdMappingDf, publicReleaseCutOff = args.publicReleaseCutOff, staging = args.staging))

def command_reviseMetadataFiles(syn, args, databaseSynIdMappingDf):
    reviseMetadataFiles(syn, args.staging, databaseSynIdMappingDf, args.genieVersion)

def reviseMetadataFiles(syn, staging, databaseSynIdMappingDf, genieVersion=None):
    ANONYMIZE_CENTER = syn.tableQuery('SELECT * FROM syn10170510')
    ANONYMIZE_CENTER_DF = ANONYMIZE_CENTER.asDataFrame()
    # if staging:
    #   parent = "syn7871696"
    # else:
    parent =  databaseSynIdMappingDf['Id'][databaseSynIdMappingDf['Database'] == 'public'].values[0]
    allFiles = syn.getChildren(parent)
    metadataEnts = [syn.get(i['id'], downloadLocation=database_to_staging.GENIE_RELEASE_DIR, ifcollision="overwrite.local") for i in allFiles if 'meta' in i['name']]
    for metaEnt in metadataEnts:
        with open(metaEnt.path, "r+") as meta:
            metaText = meta.read()
            if "meta_study" not in metaEnt.path:
                version = ''
            else:
                version = re.search(".+GENIE.+v(.+)", metaText).group(1)
            #Fix this line
            genieVersion = version if genieVersion is None else genieVersion
            dataFileVersion = re.search(".+data_(.+)[.]txt",metaText)
            if dataFileVersion is None:
                dataFileVersion = re.search(".+data_(.+)[.]seg",metaText)
            if dataFileVersion is not None:
                dataFileVersion = dataFileVersion.group(1).split("_")[-1]

            if version != genieVersion:
                metaText = metaText.replace("GENIE Cohort v%s" % version,"GENIE Cohort v%s" % genieVersion)
                metaText = metaText.replace("GENIE v%s" % version,"GENIE v%s" % genieVersion)
                if dataFileVersion is not None:
                    metaText = metaText.replace(dataFileVersion, genieVersion)
                    metaText = metaText.replace(dataFileVersion, genieVersion)
                meta.seek(0)
                meta.write(metaText)
                meta.truncate()

                storeFile(syn, metaEnt.path, parent, ANONYMIZE_CENTER_DF, genieVersion)


def createLinkVersion(syn, genie_version, caseListEntities, genePanelEntities, databaseSynIdMappingDf):
    versioning = genie_version.split(".")
    logger.info(genie_version)
    main = versioning[0]
    releaseSynId = databaseSynIdMappingDf['Id'][databaseSynIdMappingDf['Database'] == 'release'].values[0]
    publicSynId = databaseSynIdMappingDf['Id'][databaseSynIdMappingDf['Database'] == 'public'].values[0]
    #second = ".".join(versioning[1:])
    releases = synapseutils.walk(syn, releaseSynId)
    mainReleaseFolders = next(releases)[1]
    releaseFolderSynId = [synId for folderName, synId in mainReleaseFolders if folderName == "Release %s" % main] 
    if len(releaseFolderSynId) > 0:
        secondRelease = synapseutils.walk(syn, releaseFolderSynId[0])
        secondReleaseFolders = next(secondRelease)[1]
        secondReleaseFolderSynIdList = [synId for folderName, synId in secondReleaseFolders if folderName == genie_version] 
        if len(secondReleaseFolderSynIdList) > 0:
            secondReleaseFolderSynId = secondReleaseFolderSynIdList[0]
        else:
            secondReleaseFolderSynId = syn.store(synapseclient.Folder(genie_version, parent = releaseFolderSynId[0])).id
    else:
        mainReleaseFolderId = syn.store(synapseclient.Folder("Release %s" % main, parent = releaseSynId)).id
        secondReleaseFolderSynId = syn.store(synapseclient.Folder(genie_version, parent = mainReleaseFolderId)).id

    caselistId = database_to_staging.find_caselistid(syn, secondReleaseFolderSynId)

    publicRelease = syn.getChildren(publicSynId)
    [syn.store(synapseclient.Link(ents['id'], parent=secondReleaseFolderSynId, targetVersion=ents['versionNumber'])) for ents in publicRelease if ents['type'] != "org.sagebionetworks.repo.model.Folder" and ents['name'] != "data_clinical.txt"  and not ents['name'].startswith("data_gene_panel")]
    [syn.store(synapseclient.Link(ents.id, parent=caselistId, targetVersion=ents.versionNumber)) for ents in caseListEntities]
    #Store gene panels
    [syn.store(synapseclient.Link(ents.id, parent=secondReleaseFolderSynId, targetVersion=ents.versionNumber)) for ents in genePanelEntities]


if __name__ == "__main__":

    parser = argparse.ArgumentParser()
    parser.add_argument("processingDate", type=str, metavar="Jan-2017",
                        help="The process date of GENIE in Month-Year format (ie. Apr-2017)")
    parser.add_argument("cbioportalPath", type=str, metavar="/path/to/cbioportal",
                        help="Make sure you clone the cbioportal github: git clone https://github.com/cBioPortal/cbioportal.git")
    parser.add_argument("genieVersion", type=str,
                        help="GENIE public release version")    
    parser.add_argument("--publicReleaseCutOff", type=int, metavar=366, default=366,
                        help="Public release cut off time in days (Must account for leap year, 366)")
    parser.add_argument("--staging", action='store_true',
                        help="Store into staging folder")
    parser.add_argument("--test", action='store_true',
                        help="Store into staging folder")
    parser.add_argument("--pemFile", type=str, 
                        help="Path to PEM file (genie.pem)")
    parser.add_argument("--debug", action='store_true', 
                        help="Synapse debug feature")
    args = parser.parse_args()
    cbioValidatorPath = os.path.join(args.cbioportalPath,"core/src/main/scripts/importer/validateData.py")
    assert os.path.exists(cbioValidatorPath), "Please specify correct cbioportalPath"
    assert not (args.test and args.staging), "You can only specify --test or --staging, not both"
        
    syn = process_functions.synLogin(args.pemFile, debug=args.debug)
    #Get all the possible public releases
    if args.test:
        databaseSynIdMappingId = 'syn11600968'
        args.genieVersion = "TESTpublic"
    elif args.staging:
        databaseSynIdMappingId = 'syn12094210'  
    else:
        databaseSynIdMappingId = 'syn10967259'  
    databaseSynIdMapping = syn.tableQuery('select * from %s' % databaseSynIdMappingId)
    databaseSynIdMappingDf = databaseSynIdMapping.asDataFrame()
    releaseSynId = databaseSynIdMappingDf['Id'][databaseSynIdMappingDf['Database'] == 'release'].values[0]
    
    temp = synapseutils.walk(syn, releaseSynId)
    officialPublic = dict()
    for dirpath, dirnames, filenames in temp:
        release = os.path.basename(dirpath[0])
        #checkRelease = release.split(".")
        final = [i.split("-") for i in release.split(".")]
        checkRelease = []
        for i in final:
            checkRelease.extend(i)
        if args.test:
            officialPublic['TESTpublic'] = "syn12299959"
        else:
            if len(checkRelease) == 3 and checkRelease[0] != "0":
                if int(checkRelease[1]) > 0:
                    if checkRelease[0] in ['1','2']:
                        officialPublic[str(int(checkRelease[0])+1)+".0.0"] = dirpath[1]
                    else:
                        officialPublic[str(int(checkRelease[0]))+".0-public"] = dirpath[1]
    assert args.genieVersion in officialPublic.keys(), "genieVersion must be one of these: %s." % ", ".join(officialPublic.keys())
    args.releaseId = officialPublic[args.genieVersion]
    if not args.test and not args.staging:
        processTrackerSynId = databaseSynIdMappingDf['Id'][databaseSynIdMappingDf['Database'] == 'processTracker'].values[0]
        processTracker = syn.tableQuery("SELECT timeStartProcessing FROM %s where center = 'SAGE' and processingType = 'public'" % processTrackerSynId)
        processTrackerDf = processTracker.asDataFrame()
        processTrackerDf['timeStartProcessing'][0] = str(int(time.time()*1000))
        syn.store(synapseclient.Table(processTrackerSynId,processTrackerDf))

    caseListEntities, genePanelEntities = perform_consortiumToPublic(syn, args, databaseSynIdMappingDf)
    command_reviseMetadataFiles(syn, args, databaseSynIdMappingDf)
    logger.info("CBIO VALIDATION")
    #Must be exit 0 because the validator sometimes fails, but we still want to capture the output
    command = ['python',cbioValidatorPath, '-s', database_to_staging.GENIE_RELEASE_DIR, '-n','; exit 0']
    cbioOutput = subprocess.check_output(" ".join(command), shell=True)
    logger.info(cbioOutput.decode("utf-8"))
    if not args.test and not args.staging:
        log_folder_synid = databaseSynIdMappingDf['Id'][databaseSynIdMappingDf['Database'] == 'logs'].values[0]
        with open("cbioValidatorLogsPublic_%s.txt" % args.genieVersion, "w") as cbioLog:
            cbioLog.write(cbioOutput.decode("utf-8"))
        syn.store(synapseclient.File("cbioValidatorLogsPublic_%s.txt" % args.genieVersion, parentId=log_folder_synid))
        os.remove("cbioValidatorLogsPublic_%s.txt" % args.genieVersion)
    logger.info("REMOVING OLD FILES")
    process_functions.rmFiles(database_to_staging.CASE_LIST_PATH)
    if os.path.exists('%s/genie_public_meta_cna_hg19_seg.txt' % database_to_staging.GENIE_RELEASE_DIR):
        os.unlink('%s/genie_public_meta_cna_hg19_seg.txt' % database_to_staging.GENIE_RELEASE_DIR)

    logger.info("CREATING LINK VERSION")
    createLinkVersion(syn, args.genieVersion, caseListEntities, genePanelEntities, databaseSynIdMappingDf)
    #Don't update process tracker is testing or staging
    if not args.test and not args.staging:
        processTracker = syn.tableQuery("SELECT timeEndProcessing FROM %s where center = 'SAGE' and processingType = 'public'" % processTrackerSynId)
        processTrackerDf = processTracker.asDataFrame()
        processTrackerDf['timeEndProcessing'][0] = str(int(time.time()*1000))
        syn.store(synapseclient.Table(processTrackerSynId,processTrackerDf))

    if not args.test:
        logger.info("DASHBOARD UPDATE")
        dashboard_table_updater.run_dashboard(syn, databaseSynIdMappingDf, args.genieVersion, staging=args.staging, public=True)
        dashboard_markdown_html_commands = ['Rscript', os.path.join(os.path.dirname(os.path.abspath(__file__)),'dashboard_markdown_generator.R'), args.genieVersion]
        if args.staging:
            dashboard_markdown_html_commands.append('--staging')
        subprocess.check_call(dashboard_markdown_html_commands)
        logger.info("DASHBOARD UPDATE COMPLETE")

    logger.info("COMPLETED CONSORTIUM TO PUBLIC")



import os
import logging
import subprocess

import pandas as pd
import synapseclient
try:
    from synapseclient.core.exceptions import SynapseHTTPError
except ModuleNotFoundError:
    from synapseclient.exceptions import SynapseHTTPError

from .example_filetype_format import FileTypeFormat
from . import process_functions

logger = logging.getLogger(__name__)

class maf(FileTypeFormat):
    '''
    MAF file format validation / processing
    '''
    _fileType = "maf"

    _process_kwargs = [
        "processing", "path_to_GENIE", 'databaseToSynIdMappingDf',
        "vcf2mafPath", "veppath", "vepdata", 'reference']

    def _validateFilename(self, filePath):
        '''
        Validates filename.  Should be
        data_mutations_extended_CENTER.txt
        '''
        assert os.path.basename(filePath[0]) == \
            "data_mutations_extended_{}.txt".format(self.center)

    def formatMAF(self, mafDf):
        '''
        Format maf file, shortens the maf file length
        '''
        keepMafColumns = [
            'Hugo_Symbol', 'Entrez_Gene_Id', 'Center', 'NCBI_Build',
            'Chromosome', 'Start_Position', 'End_Position', 'Strand',
            'Variant_Classification', 'Variant_Type', 'Reference_Allele',
            'Tumor_Seq_Allele1', 'Tumor_Seq_Allele2', 'dbSNP_RS',
            'dbSNP_Val_Status', 'Tumor_Sample_Barcode',
            'Matched_Norm_Sample_Barcode', 'Match_Norm_Seq_Allele1',
            'Match_Norm_Seq_Allele2', 'Tumor_Validation_Allele1',
            'Tumor_Validation_Allele2', 'Match_Norm_Validation_Allele1',
            'Match_Norm_Validation_Allele2', 'Verification_Status',
            'Validation_Status', 'Mutation_Status', 'Sequencing_Phase',
            'Sequence_Source', 'Validation_Method', 'Score', 'BAM_File',
            'Sequencer', 'HGVSp_Short', 't_ref_count', 't_alt_count',
            'n_ref_count', 'n_alt_count', 'Protein_position', 'Codons',
            'SWISSPROT', 'RefSeq', 't_depth', 'n_depth', 'FILTER',
            'gnomAD_AF', 'gnomAD_AFR_AF', 'gnomAD_AMR_AF', 'gnomAD_ASJ_AF',
            'gnomAD_EAS_AF', 'gnomAD_FIN_AF', 'gnomAD_NFE_AF', 'gnomAD_OTH_AF',
            'gnomAD_SAS_AF']

        mafDf = mafDf[keepMafColumns]
        mafDf['Center'] = self.center
        mafDf['Tumor_Sample_Barcode'] = [
            process_functions.checkGenieId(i, self.center)
            for i in mafDf['Tumor_Sample_Barcode']]
        mafDf['Sequence_Source'] = float('nan')
        mafDf['Sequencer'] = float('nan')
        mafDf['Validation_Status'][
            mafDf['Validation_Status'].isin(["Unknown", "unknown"])] = ''
        return(mafDf)

    def createFinalMaf(self, mafDf, filePath, maf=False):
        '''
        Creates the maf file at the end by either
        creating the first file or appending to the existing file

        Args:
            mafDf:  Maf dataframe
            filePath: Filepath to append or create
            maf: If maf, always create the file.  Default to False.
        '''
        if not mafDf.empty:
            if os.stat(filePath).st_size == 0 or maf:
                mafset = mafDf.to_csv(sep="\t", index=False)
            else:
                mafset = mafDf.to_csv(sep="\t", index=False, header=None)
            write_or_append = "wb" if maf else "ab"
            with open(filePath, write_or_append) as maffile:
                mafSet = process_functions.removeStringFloat(mafset)
                maffile.write(mafSet.encode("utf-8"))

    def storeProcessedMaf(
            self, filePath, mafSynId, centerMafSynId, isNarrow=False):
        '''
        Stores the processed maf
        There is a isNarrow option, but note that the number of rows
        of the maf file DOES NOT change in this function

        Args:
            filePath: Path to maf file
            mafSynId: database synid
            centerMafSynid: center flat file folder synid
            isNarrow: Is the file a narrow maf. Defaul to False.
        '''
        logger.info('STORING %s' % filePath)
        database = self.syn.get(mafSynId)
        if isNarrow:
            try:
                update_table = synapseclient.Table(
                    database.id, filePath, separator="\t")
                self.syn.store(update_table)
            except SynapseTimeoutError:
                # This error occurs because of waiting for table to index.
                # Don't worry about this.
                pass
        else:
            self.syn.store(
                synapseclient.File(filePath, parentId=centerMafSynId))
        return(filePath)

    def process_steps(
            self, filePath, path_to_GENIE, databaseToSynIdMappingDf,
            vcf2mafPath, veppath, vepdata, processing, reference=None):
        '''
        Processing maf files
        '''
        if processing == self._fileType:
            mafProcessing = "mafSP" if self._fileType == "mafSP" else 'vcf2maf'
            mafSynId = databaseToSynIdMappingDf.Id[
                databaseToSynIdMappingDf['Database'] == mafProcessing][0]
            centerMafSynId = databaseToSynIdMappingDf.Id[
                databaseToSynIdMappingDf['Database'] == "centerMaf"][0]

            logger.info('MAF2MAF %s' % filePath)
            fileName = "data_mutations_extended_%s_MAF.txt" % self.center
            newMafPath = os.path.join(
                path_to_GENIE, self.center, "staging", fileName)
            narrowMafPath = os.path.join(
                path_to_GENIE, self.center, "staging",
                "data_mutations_extended_{}_MAF_narrow.txt".format(
                    self.center))
            narrowMafColumns = [
                col['name'] for col in self.syn.getTableColumns(mafSynId)
                if col['name'] != 'inBED']
            # Strips out windows indentations \r
            command = ['dos2unix', filePath]
            subprocess.check_call(command)
            tempdir = os.path.join(path_to_GENIE, self.center)
            commandCall = ["perl", os.path.join(vcf2mafPath, "maf2maf.pl"),
                           "--input-maf", filePath,
                           "--output-maf", newMafPath,
                           "--vep-fork", '8',
                           "--tmp-dir", tempdir,
                           '--vep-path', veppath,
                           '--vep-data', vepdata,
                           "--custom-enst", os.path.join(
                                vcf2mafPath,
                                "data/isoform_overrides_uniprot")]
            if reference is not None:
                # '--ref-fasta','/root/.vep/homo_sapiens/86_GRCh37/Homo_sapiens.GRCh37.75.dna.primary_assembly.fa',
                commandCall.extend(["--ref-fasta", reference])
            subprocess.check_call(commandCall)

            process_functions.rmFiles(tempdir, recursive=False)
            open(narrowMafPath, "w").close()
            if os.path.exists(newMafPath):
                # This needs to switch to streaming at some point
                mafDf = pd.read_csv(newMafPath, sep="\t", comment="#")
                mafDf.drop_duplicates(inplace=True)
                mafDf = self.formatMAF(mafDf)
                self.createFinalMaf(mafDf, newMafPath, maf=True)
                narrowMafDf = mafDf[narrowMafColumns]
                self.createFinalMaf(narrowMafDf, narrowMafPath, maf=True)
                # These functions have to be next to each other,
                # because no modifications can happen
                # Store Narrow MAF into db
                if self._fileType == "maf":
                    self.storeProcessedMaf(
                        narrowMafPath, mafSynId, centerMafSynId, isNarrow=True)
                # Store MAF flat file into synapse
                self.storeProcessedMaf(newMafPath, mafSynId, centerMafSynId)
            else:
                logger.error('ERROR PROCESSING %s' % filePath)
                filePath = "NOTPROCESSED"
        else:
            logger.info(
                "Please run with `--process {filetype}` parameter if you want "
                "to reannotate the {filetype} files".format(
                    filetype=self._fileType))
        return(filePath)

    def _validate(self, mutationDF):
        """
        This function validates the mutation file to make sure it
        adheres to the mutation SOP.

        Args:
            mutationDF: mutation dataframe

        Returns:
            Text with all the errors in the mutation file
        """

        first_header = ['CHROMOSOME', 'HUGO_SYMBOL', 'TUMOR_SAMPLE_BARCODE']
        SP = self._fileType == "mafSP"
        if SP:
            correct_column_headers = [
                'CHROMOSOME', 'START_POSITION', 'REFERENCE_ALLELE',
                'TUMOR_SAMPLE_BARCODE', 'TUMOR_SEQ_ALLELE2']
            # T_REF_COUNT + T_ALT_COUNT = T_DEPTH
        else:
            correct_column_headers = [
                'CHROMOSOME', 'START_POSITION', 'REFERENCE_ALLELE',
                'TUMOR_SAMPLE_BARCODE', 'T_ALT_COUNT', 'TUMOR_SEQ_ALLELE2']
            # T_REF_COUNT + T_ALT_COUNT = T_DEPTH
        optional_headers = \
            ['T_REF_COUNT', 'N_DEPTH', 'N_REF_COUNT', 'N_ALT_COUNT']

        mutationDF.columns = [col.upper() for col in mutationDF.columns]

        total_error = ""
        warning = ""

        # CHECK: Everything in correct_column_headers must be in mutation file
        if not all([process_functions.checkColExist(mutationDF, i)
                    for i in correct_column_headers]):
            total_error += (
                "Mutation File: "
                "Must at least have these headers: {}. "
                "If you are writing your maf file with R, please make"
                "sure to specify the 'quote=FALSE' parameter.\n".format(
                    ",".join([i for i in correct_column_headers
                              if i not in mutationDF.columns.values])))
        else:
            # CHECK: First column must be in the first_header list
            if mutationDF.columns[0] not in first_header:
                total_error += ("Mutation File: First column header must be "
                                "one of these: {}.\n".format(
                                    ", ".join(first_header)))
            # No duplicated values
            primary_cols = ['CHROMOSOME', 'START_POSITION',
                            'REFERENCE_ALLELE', 'TUMOR_SAMPLE_BARCODE',
                            'TUMOR_SEQ_ALLELE2']
            if mutationDF.duplicated(primary_cols).any():
                total_error += ("Mutation File: "
                                "Should not have duplicate rows\n")

        check_col = process_functions.checkColExist(mutationDF, "T_DEPTH")
        if not check_col and not SP:
            if not process_functions.checkColExist(mutationDF, "T_REF_COUNT"):
                total_error += (
                    "Mutation File: "
                    "If you are missing T_DEPTH, you must have T_REF_COUNT!\n")

        # CHECK: Must have either TUMOR_SEQ_ALLELE2 column
        if process_functions.checkColExist(mutationDF, "TUMOR_SEQ_ALLELE2"):
            # CHECK: The value "NA" can't be used as a placeholder
            if sum(mutationDF["TUMOR_SEQ_ALLELE2"].fillna('') == "NA") > 0:
                warning += (
                    "Mutation File: "
                    "TUMOR_SEQ_ALLELE2 column contains 'NA' values, "
                    "which cannot be placeholders for blank values.  "
                    "Please put in empty strings for blank values.\n")
            # CHECK: There can't be any null values
            if sum(mutationDF["TUMOR_SEQ_ALLELE2"].isnull()) > 0:
                total_error += (
                    "Mutation File: "
                    "TUMOR_SEQ_ALLELE2 can't have any null values.\n")

        # CHECK: Mutation file would benefit from columns in optional_headers
        if not all([
                process_functions.checkColExist(mutationDF, i)
                for i in optional_headers]) and not SP:
            warning += (
                "Mutation File: "
                "Does not have the column headers that can give extra "
                "information to the processed mutation file: {}.\n".format(
                    ", ".join([
                        i for i in optional_headers
                        if i not in mutationDF.columns.values])))

        if process_functions.checkColExist(mutationDF, "REFERENCE_ALLELE"):
            if sum(mutationDF['REFERENCE_ALLELE'] == "NA") > 0:
                warning += (
                    "Mutation File: "
                    "Your REFERENCE_ALLELE column contains NA values, "
                    "which cannot be placeholders for blank values.  "
                    "Please put in empty strings for blank values.\n")
            # CHECK: mutation file must not have empty reference
            # or variant alleles
            if sum(mutationDF['REFERENCE_ALLELE'].isnull()) > 0:
                total_error += (
                    "Mutation File: "
                    "Cannot have any empty REFERENCE_ALLELE values.\n")

        if process_functions.checkColExist(mutationDF, "CHROMOSOME"):
            # CHECK: Chromosome column can't have any values that start
            # with chr or have any WT values
            invalidValues = [
                str(i).startswith("chr") or str(i) == "WT"
                for i in mutationDF['CHROMOSOME']]
            if sum(invalidValues) > 0:
                total_error += (
                    "Mutation File: "
                    "CHROMOSOME column cannot have any values that "
                    "start with 'chr' or any 'WT' values.\n")

        return(total_error, warning)

    def _get_dataframe(self, filePathList):
        '''
        Get mutation dataframe
        '''
        mutationdf = pd.read_csv(filePathList[0],
                                 sep="\t",
                                 comment="#",
                                 na_values=['-1.#IND', '1.#QNAN', '1.#IND',
                                            '-1.#QNAN', '#N/A N/A', 'NaN',
                                            '#N/A', 'N/A', '#NA', 'NULL',
                                            '-NaN', 'nan', '-nan', ''],
                                 keep_default_na=False,
                                 # This is to check if people write files
                                 # with R, quote=T
                                 quoting=3)
        return mutationdf
